#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import json
import shutil
import subprocess
import cmind as cm
import sys


def preprocess(i):

    os_info = i['os_info']
    env = i['env']
    state = i['state']
    script_path = i['run_script_input']['path']

    rerun = True if env.get("CM_RERUN", "") != '' else False

    env['CM_MLPERF_SKIP_RUN'] = env.get('CM_MLPERF_SKIP_RUN', "no")

    mlperf_path = env['CM_MLPERF_INFERENCE_SOURCE']
    submission_checker_dir = os.path.join(mlperf_path, "tools", "submission")
    sys.path.append(submission_checker_dir)

    version = env.get('CM_MLPERF_INFERENCE_VERSION', "4.1")

    required_files = []
    required_files = get_checker_files()

    if 'CM_MLPERF_LOADGEN_SCENARIO' not in env:
        env['CM_MLPERF_LOADGEN_SCENARIO'] = "Offline"

    if 'CM_MLPERF_LOADGEN_MODE' not in env:
        print("\nNo mode given. Using accuracy as default\n")
        env['CM_MLPERF_LOADGEN_MODE'] = "accuracy"

    if env.get('OUTPUT_BASE_DIR', '') == '':
        env['OUTPUT_BASE_DIR'] = env.get(
            'CM_MLPERF_INFERENCE_RESULTS_DIR', os.getcwd())

    if 'CM_NUM_THREADS' not in env:
        if 'CM_MINIMIZE_THREADS' in env:
            env['CM_NUM_THREADS'] = str(int(env['CM_HOST_CPU_TOTAL_CORES']) //
                                        (int(env.get('CM_HOST_CPU_SOCKETS', '1')) * int(env.get('CM_HOST_CPU_TOTAL_CORES', '1'))))
        else:
            env['CM_NUM_THREADS'] = env.get('CM_HOST_CPU_TOTAL_CORES', '1')

    print("Using MLCommons Inference source from '" +
          env['CM_MLPERF_INFERENCE_SOURCE'] + "'")

    if 'CM_MLPERF_CONF' not in env:
        env['CM_MLPERF_CONF'] = os.path.join(
            env['CM_MLPERF_INFERENCE_SOURCE'], "mlperf.conf")

    RUN_CMD = ""
    state['RUN'] = {}

    scenario = env['CM_MLPERF_LOADGEN_SCENARIO']
    state['RUN'][scenario] = {}

    model_full_name = env.get('CM_ML_MODEL_FULL_NAME', env['CM_MODEL'])

    if model_full_name != env['CM_MODEL']:
        if 'model_mapping' not in state['CM_SUT_CONFIG']:
            state['CM_SUT_CONFIG']['model_mappings'] = {}
        state['CM_SUT_CONFIG']['model_mappings'][model_full_name] = env['CM_MODEL']

    if model_full_name not in i['state']['CM_SUT_CONFIG'][env['CM_SUT_NAME']]:
        i['state']['CM_SUT_CONFIG'][env['CM_SUT_NAME']][model_full_name] = {}

    if scenario not in i['state']['CM_SUT_CONFIG'][env['CM_SUT_NAME']
                                                   ][model_full_name]:
        i['state']['CM_SUT_CONFIG'][env['CM_SUT_NAME']
                                    ][model_full_name][scenario] = {}

    conf = i['state']['CM_SUT_CONFIG'][env['CM_SUT_NAME']
                                       ][model_full_name][scenario]

    mode = env['CM_MLPERF_LOADGEN_MODE']

    user_conf = ''
    if env['CM_MLPERF_RUN_STYLE'] == "fast":
        fast_factor = int(env['CM_FAST_FACTOR'])
    else:
        fast_factor = 1

    ml_model_name = env['CM_MODEL']
    if 'bert' in ml_model_name:
        ml_model_name = "bert"
    if 'dlrm' in ml_model_name:
        ml_model_name = "dlrm-v2"
    if '3d-unet' in ml_model_name:
        ml_model_name = "3d-unet"
    if 'gptj' in ml_model_name:
        ml_model_name = "gptj"
    if 'llama2-70b' in ml_model_name:
        ml_model_name = "llama2-70b"

    query_count = None

    value = None
    if scenario in ['Offline', 'Server']:
        metric = "target_qps"
        tolerance = 1.01
        # value = env.get('CM_MLPERF_LOADGEN_SERVER_TARGET_QPS') if scenario == "Server" else env.get('CM_MLPERF_LOADGEN_OFFLINE_TARGET_QPS')
        value = env.get('CM_MLPERF_LOADGEN_TARGET_QPS')
    elif scenario in ['SingleStream', 'MultiStream']:
        metric = "target_latency"
        value = env.get('CM_MLPERF_LOADGEN_TARGET_LATENCY')
        if value:
            if scenario == "SingleStream" and (
                    1000 / float(value) * 660 < 100):
                env['CM_MLPERF_USE_MAX_DURATION'] = 'no'
            elif scenario == "MultiStream" and (1000 / float(value) * 660 < 662):
                env['CM_MLPERF_USE_MAX_DURATION'] = 'no'
        if env.get('CM_MLPERF_MODEL_EQUAL_ISSUE_MODE', 'no').lower() not in ["yes", "1", "true"] and env.get(
                'CM_MLPERF_USE_MAX_DURATION', "yes").lower() not in ["no", "false", "0"]:
            tolerance = 0.4  # much lower because we have max_duration
        else:
            tolerance = 0.9
    else:
        return {'return': 1, 'error': 'Invalid scenario: {}'.format(scenario)}

    if value:
        metric_value = value
        conf[metric] = value
    else:
        if metric in conf:
            print(
                "Original configuration value {} {}".format(
                    conf[metric], metric))
            metric_value = str(
                float(
                    conf[metric]) *
                tolerance)  # some tolerance
            print(
                "Adjusted configuration value {} {}".format(
                    metric_value, metric))
        else:
            # if env.get("CM_MLPERF_FIND_PERFORMANCE_MODE", '') == "yes":
            if metric == "target_qps":
                if env.get("CM_MLPERF_FIND_PERFORMANCE_MODE", '') == "yes":
                    print("In find performance mode: using 1 as target_qps")
                else:
                    print("No target_qps specified. Using 1 as target_qps")
                conf[metric] = 1
            if metric == "target_latency":
                if env.get("CM_MLPERF_FIND_PERFORMANCE_MODE", '') == "yes":
                    print("In find performance mode: using 0.5ms as target_latency")
                else:
                    print("No target_latency specified. Using default")
                if env.get('CM_MLPERF_USE_MAX_DURATION', 'yes').lower() in ["no", "false", "0"] or env.get(
                        'CM_MLPERF_MODEL_EQUAL_ISSUE_MODE', 'no').lower() in ["yes", "1", "true"]:
                    # Total number of queries needed is a multiple of dataset
                    # size. So we dont use max_duration and so we need to be
                    # careful with the input latency
                    if '3d-unet' in env['CM_MODEL']:
                        conf[metric] = 400
                    elif 'gptj' in env['CM_MODEL']:
                        conf[metric] = 1000
                    else:
                        conf[metric] = 100
                else:
                    conf[metric] = 0.5
            metric_value = conf[metric]
            # else:
            # return {'return': 1, 'error': f"Config details missing for
            # SUT:{env['CM_SUT_NAME']}, Model:{env['CM_MODEL']}, Scenario:
            # {scenario}. Please input {metric} value"}

    # Pass the modified performance metrics to the implementation
    if env.get("CM_MLPERF_FIND_PERFORMANCE_MODE", '') == "yes":
        if metric == "target_latency" and env.get(
                'CM_MLPERF_LOADGEN_TARGET_LATENCY', '') == '':
            env['CM_MLPERF_LOADGEN_TARGET_LATENCY'] = conf[metric]
        elif metric == "target_qps" and env.get('CM_MLPERF_LOADGEN_TARGET_QPS', '') == '':
            env['CM_MLPERF_LOADGEN_TARGET_QPS'] = conf[metric]

    if env['CM_MLPERF_RUN_STYLE'] == "fast":
        if scenario == "Offline":
            metric_value = float(metric_value) / fast_factor
        if scenario in ["SingleStream", "MultiStream"]:
            metric_value = float(metric_value) * fast_factor

    elif env['CM_MLPERF_RUN_STYLE'] == "test":
        if scenario == "Offline":
            metric_value = float(env.get('CM_MLPERF_INFERENCE_TEST_QPS', 1))
        if scenario in ["SingleStream"]:
            metric_value = 1000

    elif env['CM_MLPERF_RUN_STYLE'] == "valid":
        if scenario == "Offline":
            required_min_queries_offline = {}
            required_min_queries_offline = get_required_min_queries_offline(
                env['CM_MODEL'], version)

        if mode == "compliance" and scenario == "Server":  # Adjust the server_target_qps
            test = env.get("CM_MLPERF_LOADGEN_COMPLIANCE_TEST", "TEST01")
            if test == "TEST01":
                metric_value = str(
                    float(metric_value) *
                    float(
                        env.get(
                            "CM_MLPERF_TEST01_SERVER_ADJUST_FACTOR",
                            0.96)))
            # if test == "TEST05":
            #    metric_value = str(float(metric_value) * float(env.get("CM_MLPERF_TEST05_SERVER_ADJUST_FACTOR", 0.97)))
            if test == "TEST04":
                metric_value = str(
                    float(metric_value) *
                    float(
                        env.get(
                            "CM_MLPERF_TEST04_SERVER_ADJUST_FACTOR",
                            0.97)))

    conf[metric] = metric_value
    user_conf += ml_model_name + "." + scenario + \
        "." + metric + " = " + str(metric_value) + "\n"

    if env.get('CM_MLPERF_LOADGEN_PERFORMANCE_SAMPLE_COUNT', '') != '':
        performance_sample_count = env['CM_MLPERF_LOADGEN_PERFORMANCE_SAMPLE_COUNT']
        user_conf += ml_model_name + ".*.performance_sample_count_override = " + \
            performance_sample_count + "\n"

    log_mode = mode
    if 'CM_MLPERF_POWER' in env and mode == "performance":
        log_mode = "performance_power"

    env['CM_MLPERF_INFERENCE_FINAL_RESULTS_DIR'] = os.path.join(
        env['OUTPUT_BASE_DIR'], env['CM_OUTPUT_FOLDER_NAME'])

    sut_name = env.get(
        'CM_SUT_NAME',
        env['CM_MLPERF_BACKEND'] +
        "-" +
        env['CM_MLPERF_DEVICE'])
    OUTPUT_DIR = os.path.join(env['CM_MLPERF_INFERENCE_FINAL_RESULTS_DIR'], sut_name,
                              model_full_name, scenario.lower(), mode)

    env['CM_MLPERF_INFERENCE_RESULTS_SUT_PATH'] = os.path.join(
        env['CM_MLPERF_INFERENCE_FINAL_RESULTS_DIR'], sut_name)

    if 'CM_MLPERF_POWER' in env and mode == "performance":
        env['CM_MLPERF_POWER_LOG_DIR'] = os.path.join(OUTPUT_DIR, "tmp_power")

    if mode == "accuracy":
        pass
    elif mode == "performance":
        OUTPUT_DIR = os.path.join(OUTPUT_DIR, "run_1")
    elif mode == "compliance":
        test = env.get("CM_MLPERF_LOADGEN_COMPLIANCE_TEST", "TEST01")
        OUTPUT_DIR = os.path.join(
            env['OUTPUT_BASE_DIR'],
            env['CM_OUTPUT_FOLDER_NAME'],
            sut_name,
            model_full_name,
            scenario.lower(),
            test)
        if test == "TEST01":
            audit_path = os.path.join(test, ml_model_name)
        else:
            audit_path = test

        audit_full_path = os.path.join(
            env['CM_MLPERF_INFERENCE_SOURCE'],
            "compliance",
            "nvidia",
            audit_path,
            "audit.config")
        env['CM_MLPERF_INFERENCE_AUDIT_PATH'] = audit_full_path
        # copy the audit conf to the run directory incase the implementation is
        # not supporting the audit-conf path
        if not os.path.exists(OUTPUT_DIR):
            os.makedirs(OUTPUT_DIR)
        shutil.copyfile(
            audit_full_path,
            os.path.join(
                OUTPUT_DIR,
                "audit.config"))

    env['CM_MLPERF_OUTPUT_DIR'] = OUTPUT_DIR
    env['CM_LOGS_DIR'] = OUTPUT_DIR
    env['CM_MLPERF_LOADGEN_LOGS_DIR'] = OUTPUT_DIR

    if mode == "accuracy":
        output_dir = env['CM_MLPERF_OUTPUT_DIR']
        env['CM_MLPERF_ACCURACY_RESULTS_DIR'] = output_dir
    else:
        env['CM_MLPERF_ACCURACY_RESULTS_DIR'] = ''

    run_exists = run_files_exist(log_mode, OUTPUT_DIR, required_files, env)

    if 'CM_MLPERF_POWER' in env and env.get(
            'CM_MLPERF_SHORT_RANGING_RUN', '') != 'no' and env['CM_MLPERF_RUN_STYLE'] == "valid" and mode == "performance":
        short_ranging = True
    else:
        short_ranging = False

    if short_ranging:
        import copy
        ranging_user_conf = copy.deepcopy(user_conf)
        ranging_user_conf += ml_model_name + "." + \
            scenario + ".min_duration = 300000" + "\n"

    if env['CM_MLPERF_RUN_STYLE'] == "test":
        max_duration_test_s = int(env.get('CM_MLPERF_MAX_DURATION_TEST', 30))
        max_duration_test = str(max_duration_test_s * 1000)  # in milliseconds
        query_count = int(env.get('CM_TEST_QUERY_COUNT', 5))
        min_query_count = int(
            env.get(
                'CM_MLPERF_INFERENCE_MIN_QUERY_COUNT',
                query_count))
        max_query_count = max(
            min_query_count, int(
                env.get(
                    'CM_MLPERF_INFERENCE_MAX_QUERY_COUNT', query_count)))
        user_conf += ml_model_name + "." + scenario + \
            ".max_query_count = " + str(max_query_count) + "\n"
        user_conf += ml_model_name + "." + scenario + \
            ".min_query_count = " + str(min_query_count) + "\n"
        user_conf += ml_model_name + "." + scenario + ".min_duration = 0" + "\n"
        user_conf += ml_model_name + "." + scenario + \
            ".sample_concatenate_permutation = 0" + "\n"
        env['CM_MLPERF_MAX_QUERY_COUNT'] = max_query_count

        # max_duration is effective for all scenarios except the Offline
        if env.get('CM_MLPERF_USE_MAX_DURATION', 'yes').lower() not in [
                "no", "false", "0"]:
            if scenario != "Offline":
                user_conf += ml_model_name + "." + scenario + \
                    f".max_duration = {max_duration_test}" + "\n"

    elif env['CM_MLPERF_RUN_STYLE'] == "fast":
        user_conf += ml_model_name + "." + scenario + \
            ".sample_concatenate_permutation = 0" + "\n"
        max_duration_fast_s = int(env.get('CM_MLPERF_MAX_DURATION_FAST', 120))
        max_duration_fast = str(max_duration_fast_s * 1000)  # in milliseconds
        if scenario == "Server":
            user_conf += ml_model_name + "." + scenario + \
                f".max_duration = {max_duration_fast}" + "\n"
            target_qps = conf['target_qps']
            query_count = str(int((660 / fast_factor) * (float(target_qps))))
            user_conf += ml_model_name + "." + scenario + \
                ".max_query_count = " + query_count + "\n"
            env['CM_MLPERF_MAX_QUERY_COUNT'] = query_count

    else:
        max_duration_valid_s = int(
            env.get('CM_MLPERF_MAX_DURATION_VALID', 660))
        max_duration_valid = str(
            max_duration_valid_s *
            1000)  # in milliseconds
        max_duration_ranging_s = int(
            env.get('CM_MLPERF_MAX_DURATION_RANGING', 300))
        max_duration_ranging = str(
            max_duration_ranging_s *
            1000)  # in milliseconds
        if scenario == "MultiStream" or scenario == "SingleStream":
            if env.get('CM_MLPERF_USE_MAX_DURATION', 'yes').lower() not in ["no", "false", "0"] and env.get(
                    'CM_MLPERF_MODEL_EQUAL_ISSUE_MODE', 'no').lower() not in ["yes", "1", "true"]:
                user_conf += ml_model_name + "." + scenario + \
                    f".max_duration = {max_duration_valid}" + "\n"
            elif env.get('CM_MLPERF_INFERENCE_MIN_DURATION', '') != '':
                user_conf += ml_model_name + "." + scenario + ".min_duration = " + \
                    env['CM_MLPERF_INFERENCE_MIN_DURATION'] + " \n"
            if scenario == "MultiStream":
                user_conf += ml_model_name + "." + scenario + ".min_query_count = " + \
                    env.get(
                        'CM_MLPERF_INFERENCE_MULTISTREAM_MIN_QUERY_COUNT',
                        "662") + "\n"
            if short_ranging:
                ranging_user_conf += ml_model_name + "." + scenario + \
                    f".max_duration = {max_duration_ranging} \n "
        elif scenario == "Offline":
            query_count = int(float(conf['target_qps']) * 660)
            query_count = str(max(query_count, required_min_queries_offline))

            # user_conf += ml_model_name + "." + scenario + ".max_query_count = " + str(int(query_count)+40) + "\n"
            if short_ranging:
                ranging_query_count = str(int(float(conf['target_qps']) * 300))
                ranging_user_conf += ml_model_name + "." + scenario + \
                    ".max_query_count = " + str(ranging_query_count) + "\n"
                ranging_user_conf += ml_model_name + "." + scenario + ".min_query_count = 0 \n"

    if query_count:
        # needed for squad accuracy checker
        env['CM_MAX_EXAMPLES'] = str(query_count)

    import uuid
    from pathlib import Path
    key = uuid.uuid4().hex
    user_conf_path = os.path.join(script_path, "tmp", key + ".conf")
    user_conf_file = Path(user_conf_path)
    user_conf_file.parent.mkdir(exist_ok=True, parents=True)
    user_conf_file.write_text(user_conf)

    if short_ranging:
        ranging_user_conf_path = os.path.join(
            script_path, "tmp", "ranging_" + key + ".conf")
        ranging_user_conf_file = Path(ranging_user_conf_path)
        ranging_user_conf_file.write_text(ranging_user_conf)

    if (env.get('CM_MLPERF_LOADGEN_QUERY_COUNT', '') == '') and query_count and (
            (mode != "accuracy") or (env['CM_MLPERF_RUN_STYLE'] != "valid")):
        env['CM_MLPERF_LOADGEN_QUERY_COUNT'] = str(query_count)

    if not run_exists or rerun:

        print("Output Dir: '" + OUTPUT_DIR + "'")
        print(user_conf)
        if env.get('CM_MLPERF_POWER', '') == "yes" and os.path.exists(
                env.get('CM_MLPERF_POWER_LOG_DIR', '')):
            shutil.rmtree(env['CM_MLPERF_POWER_LOG_DIR'])
    else:
        if not env.get('CM_MLPERF_COMPLIANCE_RUN_POSTPONED', False):
            print("Run files exist, skipping run...\n")
        env['CM_MLPERF_SKIP_RUN'] = "yes"

    if not run_exists or rerun or not measure_files_exist(OUTPUT_DIR,
                                                          required_files[4]) or env.get("CM_MLPERF_LOADGEN_COMPLIANCE", "") == "yes" or env.get("CM_REGENERATE_MEASURE_FILES", False):

        env['CM_MLPERF_TESTING_USER_CONF'] = os.path.join(
            os.path.dirname(user_conf_path), key + ".conf")  # user_conf_path
        env['CM_MLPERF_RANGING_USER_CONF'] = os.path.join(
            os.path.dirname(user_conf_path),
            "ranging_" + key + ".conf")  # ranging_user_conf_path for a shorter run

        if short_ranging:
            env['CM_MLPERF_USER_CONF'] = r"\${CM_MLPERF_USER_CONF}"
        else:
            env['CM_MLPERF_USER_CONF'] = os.path.join(
                os.path.dirname(user_conf_path), key + ".conf")  # user_conf_path
    else:
        print(
            f"Measure files exist at {OUTPUT_DIR}. Skipping regeneration...\n")
        env['CM_MLPERF_USER_CONF'] = ''

    os.makedirs(OUTPUT_DIR, exist_ok=True)

    if str(env.get('CM_MLPERF_RESULTS_DIR_SHARED', '')
           ).lower() in ["yes", "true", "1"]:
        os.chmod(OUTPUT_DIR, 0o2775)

    return {'return': 0}


def run_files_exist(mode, OUTPUT_DIR, run_files, env):
    import submission_checker as checker
    from log_parser import MLPerfLog

    is_valid = True

    file_loc = {
        "accuracy": 0,
        "performance": 1,
        "power": 2,
        "performance_power": 3,
        "measure": 4,
        "compliance": 1}

    required_files = run_files[file_loc[mode]]
    if mode == "performance_power":
        for file_ in run_files[2]:
            file_path = os.path.join(
                os.path.dirname(OUTPUT_DIR), "power", file_)
            if (not os.path.exists(file_path)
                    or os.stat(file_path).st_size == 0):
                return False
        # We need performance files too in the run directory
        required_files += run_files[1]

    for file_ in required_files:
        file_path = os.path.join(OUTPUT_DIR, file_)
        if (not os.path.exists(file_path) or os.stat(
                file_path).st_size == 0) and file_ != "accuracy.txt":
            return False

        if file_ == "mlperf_log_detail.txt" and "performance" in mode:
            mlperf_log = MLPerfLog(file_path)
            if (
                "result_validity" not in mlperf_log.get_keys()
                or mlperf_log["result_validity"] != "VALID"
            ):
                return False

    if mode == "compliance":
        # If a performance run followed the last compliance run, compliance
        # check needs to be redone
        RESULT_DIR = os.path.split(OUTPUT_DIR)[0]
        COMPLIANCE_DIR = OUTPUT_DIR
        OUTPUT_DIR = os.path.dirname(COMPLIANCE_DIR)

        # If reference test result is invalid, don't do compliance run
        file_path = os.path.join(
            RESULT_DIR,
            "performance",
            "run_1",
            "mlperf_log_detail.txt")
        mlperf_log = MLPerfLog(file_path)
        if (
            "result_validity" not in mlperf_log.get_keys()
            or mlperf_log["result_validity"] != "VALID"
        ):
            env['CM_MLPERF_COMPLIANCE_RUN_POSTPONED'] = True
            return True

        test = env['CM_MLPERF_LOADGEN_COMPLIANCE_TEST']

        SCRIPT_PATH = os.path.join(
            env['CM_MLPERF_INFERENCE_SOURCE'],
            "compliance",
            "nvidia",
            test,
            "run_verification.py")
        if test == "TEST06":
            cmd = f"{env['CM_PYTHON_BIN_WITH_PATH']}  {SCRIPT_PATH}  -c  {COMPLIANCE_DIR}  -o  {OUTPUT_DIR} --scenario {scenario} --dtype int32"
        else:
            cmd = f"{env['CM_PYTHON_BIN_WITH_PATH']}  {SCRIPT_PATH}  -r {RESULT_DIR} -c  {COMPLIANCE_DIR}  -o  {OUTPUT_DIR}"

        print(cmd)
        os.system(cmd)

        is_valid = checker.check_compliance_perf_dir(COMPLIANCE_DIR)

        if not is_valid and 'Stream' in env['CM_MLPERF_LOADGEN_SCENARIO']:
            # We have the determined latency, compliance test failed, so lets
            # not use max duration
            env['CM_MLPERF_USE_MAX_DURATION'] = 'no'
            env['CM_MLPERF_INFERENCE_MIN_DURATION'] = '990000'  # Try a longer run

        return is_valid

    if "power" in mode and env.get(
            'CM_MLPERF_SKIP_POWER_CHECKS', 'no').lower() not in ["yes", "true", "on"]:
        from power.power_checker import check as check_power_more
        try:
            is_valid = check_power_more(os.path.dirname(OUTPUT_DIR)) == 0
        except BaseException:
            is_valid = False
        return is_valid

    return is_valid


def measure_files_exist(OUTPUT_DIR, run_files):
    for file in run_files:
        file_path = os.path.join(OUTPUT_DIR, file)
        if not os.path.exists(file_path):
            return False
    return True


def get_checker_files():
    import submission_checker as checker

    REQUIRED_ACC_FILES = checker.REQUIRED_ACC_FILES
    REQUIRED_PERF_FILES = checker.REQUIRED_PERF_FILES
    REQUIRED_POWER_FILES = checker.REQUIRED_POWER_FILES
    REQUIRED_PERF_POWER_FILES = checker.REQUIRED_PERF_POWER_FILES
    REQUIRED_MEASURE_FILES = checker.REQUIRED_MEASURE_FILES
    return REQUIRED_ACC_FILES, REQUIRED_PERF_FILES, REQUIRED_POWER_FILES, REQUIRED_PERF_POWER_FILES, REQUIRED_MEASURE_FILES


def get_required_min_queries_offline(model, version):

    import submission_checker as checker

    version_split = version.split(".")
    if int(version[0]) < 4:
        return 24756

    REQUIRED_MIN_QUERIES = checker.OFFLINE_MIN_SPQ_SINCE_V4
    mlperf_model = model
    mlperf_model = mlperf_model.replace("resnet50", "resnet")

    return REQUIRED_MIN_QUERIES[mlperf_model]



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import cmind as cm
import os
import shutil
import json


def preprocess(i):

    os_info = i['os_info']
    env = i['env']
    script_dir = i['run_script_input']['path']
    config_dir = os.path.join(
        script_dir, env.get(
            'CM_TERRAFORM_CONFIG_DIR_NAME', ''))
    env['CM_TERRAFORM_CONFIG_DIR'] = config_dir
    cache_dir = os.getcwd()

    print(f"Running terraform from {cache_dir}")

    shutil.copy(os.path.join(config_dir, "main.tf"), cache_dir)
    env['CM_TERRAFORM_RUN_DIR'] = cache_dir

    return {'return': 0}


def postprocess(i):
    env = i['env']
    if env.get('CM_DESTROY_TERRAFORM'):
        return {'return': 0}
    state = i['state']
    with open("terraform.tfstate") as f:
        tfstate = json.load(f)
#    print(tfstate)
    resources = tfstate['resources']
    for resource in resources:
        if resource['type'] == 'aws_instance':
            aws_resource = resource
            break
    instances_state = aws_resource['instances']
    state['CM_TF_NEW_INSTANCES_STATE'] = []
    ssh_key_file = env.get('CM_SSH_KEY_FILE')
    user = 'ubuntu'
    for instance_state in instances_state:
        instance_attributes = instance_state['attributes']
        state['CM_TF_NEW_INSTANCES_STATE'].append(instance_attributes)
        public_ip = instance_attributes['public_ip']
        if env.get('CM_TERRAFORM_CM_INIT'):
            run_input = {
                'automation': 'script',
                'action': 'run',
                'tags': 'remote,run,ssh',
                'env': {
                },
                'host': public_ip,
                'user': user,
                'skip_host_verify': True,
                'ssh_key_file': ssh_key_file,
                'quiet': True,
                'silent': True,
                'run_cmds': [
                    "sudo apt-get update",
                    "sudo apt-get -y upgrade",
                    "sudo apt-get install -y python3-pip",
                    "python3 -m pip install cmind",
                    "source ~/.profile",
                    "cm pull repo ctuning@mlcommons-ck",
                    "cm run script --tags=get,sys-utils-cm"
                ]
            }
            if env.get('CM_TERRAFORM_RUN_COMMANDS'):
                run_cmds = env.get('CM_TERRAFORM_RUN_COMMANDS')
                for cmd in run_cmds:
                    cmd = cmd.replace(":", "=")
                    cmd = cmd.replace(";;", ",")
                    run_input['run_cmds'].append(cmd)
            r = cm.access(run_input)
            if r['return'] > 0:
                return r
            # print(r)
        print_attr(instance_attributes, "id")
        print_attr(instance_attributes, "instance_type")
        print_attr(instance_attributes, "public_ip")
        print_attr(instance_attributes, "public_dns")
        print_attr(instance_attributes, "security_groups")

    return {'return': 0}


def print_attr(instance_attributes, key):
    if key in instance_attributes:
        print(key.upper() + ": " + str(instance_attributes[key]))



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):
    os_info = i['os_info']
    if os_info['platform'] == 'windows':
        return {'return': 1, 'error': 'Windows is not supported in this script yet'}

    env = i['env']

    # env['CM_GIT_CHECKOUT'] = env['CM_TMP_GIT_BRANCH_NAME']

    return {'return': 0}


def postprocess(i):

    env = i['env']

    paths = [
        "+C_INCLUDE_PATH",
        "+CPLUS_INCLUDE_PATH",
        "+LD_LIBRARY_PATH",
        "+DYLD_FALLBACK_LIBRARY_PATH"
    ]

    for key in paths:
        env[key] = []

    include_paths = [env['CM_TMP_CURRENT_SCRIPT_PATH']]

    for inc_path in include_paths:
        env['+C_INCLUDE_PATH'].append(inc_path)
        env['+CPLUS_INCLUDE_PATH'].append(inc_path)

    version = "master"
    env['CM_QAIC_API_SRC_FILE'] = os.path.join(
        env['CM_TMP_CURRENT_SCRIPT_PATH'], version, "QAicInfApi.cpp")
    env['CM_QAIC_API_INC_FILE'] = os.path.join(
        env['CM_TMP_CURRENT_SCRIPT_PATH'], version, "QAicInfApi.h")

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']
    automation = i['automation']
    meta = i['meta']
    os_info = i['os_info']
    if os_info['platform'] == 'windows':
        return {'return': 0}

    env['CM_DATASET_IMAGENET_TRAIN_REQUIRE_DAE'] = 'no'

    path = env.get('CM_INPUT', env.get('IMAGENET_TRAIN_PATH', '')).strip()

    if path == '':
        if env.get('CM_DATASET_IMAGENET_TRAIN_TORRENT_PATH'):
            path = env['CM_DATASET_IMAGENET_TRAIN_TORRENT_PATH']
            env['CM_DAE_EXTRA_TAGS'] = "_torrent"
            env['CM_DAE_TORRENT_PATH'] = path
            env['CM_DATASET_IMAGENET_VAL_REQUIRE_DAE'] = 'yes'

            return {'return': 0}

        else:
            return {'return': 1, 'error': 'Please rerun the last CM command with --env.IMAGENET_TRAIN_PATH={path the folder containing full ImageNet training images} or envoke cm run script "get train dataset imagenet" --input={path to the folder containing ImageNet training images}'}

    elif not os.path.isdir(path):
        if path.endswith(".tar"):
            # env['CM_DAE_FILEPATH'] = path
            env['CM_EXTRACT_FILEPATH'] = path
            env['CM_DAE_ONLY_EXTRACT'] = 'yes'
            return {'return': 0}
        else:
            return {'return': 1,
                    'error': 'Path {} doesn\'t exist'.format(path)}
    else:
        env['CM_EXTRACT_EXTRACTED_PATH'] = path

    return {'return': 0}


def postprocess(i):

    os_info = i['os_info']
    if os_info['platform'] == 'windows':
        return {'return': 0}

    env = i['env']

    path = env['CM_EXTRACT_EXTRACTED_PATH']

    path_tar = os.path.join(path, 'n01440764.tar')

    if not os.path.isfile(path_tar):
        return {'return': 1,
                'error': 'ImageNet file {} not found'.format(path_tar)}

    env['CM_DATASET_PATH'] = path
    env['CM_DATASET_IMAGENET_PATH'] = path
    env['CM_DATASET_IMAGENET_TRAIN_PATH'] = path

    env['CM_GET_DEPENDENT_CACHED_PATH'] = path

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    recursion_spaces = i['recursion_spaces']

    cur_dir = os.getcwd()

    os_info = i['os_info']

    env = i['env']

    if str(env.get('CUDA_SKIP_SUDO', '')).lower() == 'true':
        env['CM_SUDO'] = ''

    meta = i['meta']
    automation = i['automation']
    version = env.get('CM_VERSION')

    supported_versions = list(meta['versions'].keys())

    if version not in supported_versions:
        return {'return': 1, 'error': "Only cuDNN versions {} are supported now".format(
            ', '.join(supported_versions))}

    env['CM_CUDNN_VERSION'] = version

    filename = env['CM_CUDNN_TAR_FILE_NAME_TEMPLATE']
    cudnn_md5sum = env.get('CM_CUDNN_TAR_MD5SUM', '')

    cuda_version_split = env['CM_CUDA_VERSION'].split('.')
    cuda_version_major = cuda_version_split[0]

    filename = filename.replace('{{CUDA_MAJOR_VERSION}}', cuda_version_major)

    env['CM_CUDNN_TAR_FILE_NAME'] = filename

    cudnn_dir = filename[:-7]

    cudnn_url = f'https://developer.download.nvidia.com/compute/cudnn/redist/cudnn/linux-x86_64/{filename}'

    print('')
    print(f'URL to download cuDNN: {cudnn_url}')

    env['CM_CUDNN_TAR_DIR'] = cudnn_dir
    env['CM_CUDNN_UNTAR_PATH'] = os.path.join(cur_dir, cudnn_dir)
    env['WGET_URL'] = cudnn_url
    env['CM_DOWNLOAD_CHECKSUM'] = cudnn_md5sum

    return {'return': 0}



import typing

import numpy as np
import onnx
import onnxruntime as ort

from loadgen.model import Model, ModelFactory, ModelInput, ModelInputSampler

xinput = input

ONNX_TO_NP_TYPE_MAP = {
    "tensor(bool)": bool,
    "tensor(int)": np.int32,
    "tensor(int32)": np.int32,
    "tensor(int8)": np.int8,
    "tensor(uint8)": np.uint8,
    "tensor(int16)": np.int16,
    "tensor(uint16)": np.uint16,
    "tensor(uint64)": np.uint64,
    "tensor(int64)": np.int64,
    "tensor(float16)": np.float16,
    "tensor(float)": np.float32,
    "tensor(double)": np.float64,
    "tensor(string)": np.string_,
}


class XModel(Model):
    def __init__(self, session: ort.InferenceSession):
        assert session is not None
        self.session = session

    def predict(self, input: ModelInput):
        output = self.session.run(None, input)
        return output


class XModelFactory(ModelFactory):
    def __init__(
        self,
        model_path: str,
        execution_provider="CPUExecutionProvider",
        execution_mode="",
        intra_op_threads=0,
        inter_op_threads=0,
        model_code='',         # Not used here
        model_cfg={},          # Not used here
        model_sample_pickle=''  # Not used here
    ):
        self.model_path = model_path
        self.execution_provider = execution_provider
        self.session_options = ort.SessionOptions()
        if execution_mode.lower() == "sequential":
            self.session_options.execution_mode = ort.ExecutionMode.ORT_SEQUENTIAL
        elif execution_mode.lower() == "parallel":
            self.session_options.execution_mode = ort.ExecutionMode.ORT_PARALLEL
        self.session_options.intra_op_num_threads = intra_op_threads
        self.session_options.inter_op_num_threads = inter_op_threads

    def create(self) -> Model:
        print('Loading model: {}'.format(self.model_path))
#        model = onnx.load(self.model_path)
        session_eps = [self.execution_provider]
        session = ort.InferenceSession(
            # model.SerializeToString(), self.session_options,
            # providers=session_eps
            self.model_path, self.session_options, providers=session_eps
        )
        return XModel(session)


class XModelInputSampler(ModelInputSampler):
    def __init__(self, model_factory: XModelFactory):
        model = model_factory.create()
        input_defs = model.session.get_inputs()
        self.inputs: typing.Dict[str,
                                 typing.Tuple[np.dtype,
                                              typing.List[int]]] = dict()
        for input in input_defs:
            input_name = input.name
            input_type = ONNX_TO_NP_TYPE_MAP[input.type]
            input_dim = [
                1 if (x is None or (isinstance(x, str))) else x for x in input.shape
            ]
            self.inputs[input_name] = (input_type, input_dim)

    def sample(self, id_: int) -> ModelInput:
        input = dict()
        for name, spec in self.inputs.items():
            val = np.random.random_sample(spec[1]).astype(spec[0])
            input[name] = val
        return input



"""
Pytoch native backend
Extended by Grigori Fursin for the ABTF demo
"""
# pylint: disable=unused-argument,missing-docstring
import torch  # currently supports pytorch1.0
import torchvision
import backend

import os
import sys
import importlib


class BackendPytorchNative(backend.Backend):
    def __init__(self):
        super(BackendPytorchNative, self).__init__()
        self.sess = None
        self.model = None
        self.device = "cuda:0" if torch.cuda.is_available() else "cpu"

        # Grigori added for ABTF model
        self.config = None
        self.num_classes = None
        self.image_size = None

    def version(self):
        return torch.__version__

    def name(self):
        return "pytorch-native"

    def image_format(self):
        return "NCHW"

    def load(self, model_path, inputs=None, outputs=None):

        # From ABTF code
        sys.path.insert(0, os.environ['CM_ML_MODEL_CODE_WITH_PATH'])

        from src.transform import SSDTransformer
        from src.utils import generate_dboxes, Encoder, colors, coco_classes
        from src.model import SSD, ResNet

        abtf_model_config = os.environ.get('CM_ABTF_ML_MODEL_CONFIG', '')

        num_classes_str = os.environ.get('CM_ABTF_NUM_CLASSES', '').strip()
        self.num_classes = int(
            num_classes_str) if num_classes_str != '' else 15

        self.config = importlib.import_module('config.' + abtf_model_config)
        self.image_size = self.config.model['image_size']

        self.model = SSD(
            self.config.model,
            backbone=ResNet(
                self.config.model),
            num_classes=self.num_classes)

        checkpoint = torch.load(
            model_path,
            map_location=torch.device(
                self.device))

        self.model.load_state_dict(checkpoint["model_state_dict"])

        if self.device.startswith('cuda'):
            self.model.cuda()

        self.model.eval()

        self.model = self.model.to(self.device)

        self.inputs = inputs
        self.outputs = outputs

        return self

    def predict(self, feed):
        # For ABTF

        # Always first element for now (later may stack for batching)
        img = feed['image'][0]

        if torch.cuda.is_available():
            img = img.cuda()

        inp = img.unsqueeze(dim=0)

        with torch.no_grad():
            ploc, plabel = self.model(inp)

            output = (ploc, plabel)

        return output



from cmind import utils
import os
import json
import shutil
import subprocess
import mlperf_utils
from log_parser import MLPerfLog


def preprocess(i):

    os_info = i['os_info']
    env = i['env']
    state = i['state']
    script_path = i['run_script_input']['path']

    if 'cmd' in i['input']:
        state['mlperf_inference_run_cmd'] = "cm run script " + \
            " ".join(i['input']['cmd'])

    state['mlperf-inference-implementation'] = {}

    run_state = i['run_script_input']['run_state']
    state['mlperf-inference-implementation']['script_id'] = run_state['script_id'] + \
        ":" + ",".join(run_state['script_variation_tags'])

    return {'return': 0}


def postprocess(i):

    env = i['env']
    state = i['state']

    inp = i['input']
    os_info = i['os_info']

    xsep = '^' if os_info['platform'] == 'windows' else '\\'

    env['CMD'] = ''

    # if env.get('CM_MLPERF_USER_CONF', '') == '':
    #    return {'return': 0}

    output_dir = env['CM_MLPERF_OUTPUT_DIR']
    mode = env['CM_MLPERF_LOADGEN_MODE']

    model = env['CM_MODEL']
    model_full_name = env.get('CM_ML_MODEL_FULL_NAME', model)

    scenario = env['CM_MLPERF_LOADGEN_SCENARIO']

    if not os.path.exists(output_dir) or not os.path.exists(
            os.path.join(output_dir, "mlperf_log_summary.txt")):
        # No output, fake_run?
        return {'return': 0}

    mlperf_log = MLPerfLog(os.path.join(output_dir, "mlperf_log_detail.txt"))
    if mode == "performance":
        result = mlperf_log['result_mean_latency_ns'] / 1000000
    elif mode == "accuracy":
        if not env.get(
                'CM_COGNATA_ACCURACY_DUMP_FILE'):  # can happen while reusing old runs
            env['CM_COGNATA_ACCURACY_DUMP_FILE'] = os.path.join(
                output_dir, "accuracy.txt")
        acc = ""
        if os.path.exists(env['CM_COGNATA_ACCURACY_DUMP_FILE']):
            with open(env['CM_COGNATA_ACCURACY_DUMP_FILE'], "r") as f:
                acc = f.readline()
        result = acc
    else:
        return {'return': 1, 'error': f"Unknown mode {mode}"}

    valid = {'performance': True, 'accuracy': True}  # its POC
    power_result = None  # No power measurement in POC

    # result, valid, power_result = mlperf_utils.get_result_from_log(env['CM_MLPERF_LAST_RELEASE'], model, scenario, output_dir, mode)

    if not state.get('cm-mlperf-inference-results'):
        state['cm-mlperf-inference-results'] = {}
    if not state.get('cm-mlperf-inference-results-last'):
        state['cm-mlperf-inference-results-last'] = {}
    if not state['cm-mlperf-inference-results'].get(
            state['CM_SUT_CONFIG_NAME']):
        state['cm-mlperf-inference-results'][state['CM_SUT_CONFIG_NAME']] = {}
    if not state['cm-mlperf-inference-results'][state['CM_SUT_CONFIG_NAME']
                                                ].get(model):
        state['cm-mlperf-inference-results'][state['CM_SUT_CONFIG_NAME']][model] = {}
    if not state['cm-mlperf-inference-results'][state['CM_SUT_CONFIG_NAME']
                                                ][model].get(scenario):
        state['cm-mlperf-inference-results'][state['CM_SUT_CONFIG_NAME']
                                             ][model][scenario] = {}

    state['cm-mlperf-inference-results'][state['CM_SUT_CONFIG_NAME']
                                         ][model][scenario][mode] = result
    state['cm-mlperf-inference-results'][state['CM_SUT_CONFIG_NAME']
                                         ][model][scenario][mode + '_valid'] = valid.get(mode, False)

    state['cm-mlperf-inference-results-last'][mode] = result
    state['cm-mlperf-inference-results-last'][mode +
                                              '_valid'] = valid.get(mode, False)

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil


def preprocess(i):

    os_info = i['os_info']

    if os_info['platform'] == 'windows':
        return {'return': 1, 'error': 'Windows is not supported in this script yet'}
    env = i['env']

    if env.get('CM_MLPERF_SKIP_RUN', '') == "yes":
        return {'return': 0}

    env['CM_MLPERF_AMD_SCRIPT_PATH'] = env['CM_TMP_CURRENT_SCRIPT_PATH']
    env['CM_MLPERF_AMD_CODE_PATH'] = os.path.join(
        env['CM_MLPERF_INFERENCE_IMPLEMENTATION_REPO'], "closed", "AMD")

    if 'CM_MODEL' not in env:
        return {
            'return': 1, 'error': 'Please select a variation specifying the model to run'}
    if 'CM_MLPERF_BACKEND' not in env:
        return {'return': 1,
                'error': 'Please select a variation specifying the backend'}
    if 'CM_MLPERF_DEVICE' not in env:
        return {
            'return': 1, 'error': 'Please select a variation specifying the device to run on'}

    if "llama2" in env['CM_MODEL']:
        env['CM_RUN_DIR'] = i['run_script_input']['path']
        env['CM_MLPERF_AMD_LLAMA2_CODE_PATH'] = os.path.join(
            env['CM_MLPERF_AMD_CODE_PATH'], "llama2-70b-99.9/VllmFp8")
        env['CM_RUN_CMD'] = "bash run-llama2.sh "
    else:
        return {'return': 1, 'error': 'Model {} not supported'.format(
            env['CM_MODEL'])}

    return {'return': 0}
    # return {'return':1, 'error': 'Run command needs to be tested'}


def postprocess(i):

    env = i['env']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):
    os_info = i['os_info']

    env = i['env']

    # Check if input files are empty and add files
    input_file_1 = env.get('CM_INPUT_1', '')
    if input_file_1 == '':
        input_file_1 = 'ipol-paper-2024-439-sample-image-1.png'

    if not os.path.isfile(input_file_1):
        return {'return': 1,
                'error': 'input file 1 "{}" not found'.format(input_file_1)}

    env['CM_INPUT_1'] = os.path.abspath(input_file_1)

    input_file_2 = env.get('CM_INPUT_2', '')
    if input_file_2 == '':
        input_file_2 = 'ipol-paper-2024-439-sample-image-2.png'

    if not os.path.isfile(input_file_2):
        return {'return': 1,
                'error': 'input file 2 "{}" not found'.format(input_file_2)}

    env['CM_INPUT_2'] = os.path.abspath(input_file_2)

    return {'return': 0}


def postprocess(i):

    print('')
    print('Please check "diff.png"')
    print('')

    return {'return': 0}



# Universal cfg for CM automations
#
# Author: Grigori Fursin
# Contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

import os

from cmind.automation import Automation
from cmind import utils


class CAutomation(Automation):
    """
    Automation actions
    """

    ############################################################
    def __init__(self, cmind, automation_file):
        super().__init__(cmind, __file__)

    ############################################################
    def test(self, i):
        """
        Test automation

        Args:
          (CM input dict):

          (out) (str): if 'con', output to console

          automation (str): automation as CM string object

          parsed_automation (list): prepared in CM CLI or CM access function
                                    [ (automation alias, automation UID) ] or
                                    [ (automation alias, automation UID), (automation repo alias, automation repo UID) ]

          (artifact) (str): artifact as CM string object

          (parsed_artifact) (list): prepared in CM CLI or CM access function
                                    [ (artifact alias, artifact UID) ] or
                                    [ (artifact alias, artifact UID), (artifact repo alias, artifact repo UID) ]

          ...

        Returns:
          (CM return dict):

          * return (int): return code == 0 if no error and >0 if error
          * (error) (str): error string if return>0

          * Output from this automation action

        """

        import json
        print(json.dumps(i, indent=2))

        return {'return': 0}

    ############################################################
    def xset(self, i):
        """
        Set keys in configuration

        Args:
          (CM input dict):

            (out) (str): if 'con', output to console

            (artifact) (str): CM artifact with configuration
            (tags) (str): list of tags to find CM artifact with configuration

            (key) (dict): updating config
          ...

        Returns:
          (CM return dict):

          * return (int): return code == 0 if no error and >0 if error
          * (error) (str): error string if return>0

          * Output from this automation action

        """

        import json

        r = self._find_cfg_artifact(i)
        if r['return'] > 0:
            return r

        # Path to cfg
        path = r['path']
        path_to_config = r['path_to_config']
        config = r['config']

        # Clean input to leave only keys for the configuration
        new_config = i.get('key', {})

        # If new config is empty, just print existing config
        if len(new_config) > 0:
            # Check if need to delete some
            def check_to_delete(d):

                for k in list(d.keys()):
                    v = d[k]
                    if isinstance(v, dict):
                        check_to_delete(v)
                    else:
                        if k.endswith('-'):
                            if k[:-1] in d:
                                del (d[k[:-1]])
                            del (d[k])
                        else:
                            vsl = str(v).lower()
                            if vsl == 'none':
                                v = None
                            elif vsl == 'false':
                                v = False
                            elif vsl == 'true':
                                v = True

                            d[k] = v

            utils.merge_dicts({'dict1': config,
                               'dict2': new_config,
                               'append_lists': True,
                               'append_unique': True})

            check_to_delete(config)

            r = utils.save_json(path_to_config, config)
            if r['return'] > 0:
                return r

        # Print config
        print('Config:')
        print('')
        print(json.dumps(config, indent=2))

        return {'return': 0}

    ############################################################
    def load(self, i):
        """
        Load configuration

        Args:
          (CM input dict):

            (out) (str): if 'con', output to console

            (artifact) (str): CM artifact with configuration
            (tags) (str): list of tags to find CM artifact with configuration
          ...

        Returns:
          (CM return dict):

          * return (int): return code == 0 if no error and >0 if error
          * (error) (str): error string if return>0

          * Output from this automation action

        """

        return self._find_cfg_artifact(i)

    ############################################################
    def _find_cfg_artifact(self, i):
        """
        Args:
          (CM input dict):

            (out) (str): if 'con', output to console

            (artifact) (str): CM artifact with configuration
            (tags) (str): list of tags to find CM artifact with configuration
          ...

        Returns:
          (CM return dict):

          * return (int): return code == 0 if no error and >0 if error
          * (error) (str): error string if return>0

          * Output from this automation action

        """

        # Clean input to find artifact
        ii = utils.sub_input(i, self.cmind.cfg['artifact_keys'] + ['tags'])

        parsed_artifact = i.get('parsed_artifact', [])

        artifact_obj = parsed_artifact[0] if len(parsed_artifact) > 0 else None
        artifact_repo = parsed_artifact[1] if len(
            parsed_artifact) > 1 else None

        artifact = i.get('artifact', '')

        if artifact == '':
            ii['artifact'] = 'default'

        tags = ii.get('tags', '')

        if 'cm-universal-cfg' not in tags:
            if tags != '':
                tags += ','
            tags += 'cm-universal-cfg'

        ii['tags'] = tags

        automation = ii['automation']
        if automation != '.' and ',' not in automation:
            ii['automation'] = automation + ',' + self.meta['uid']

        # Add placeholder (use common action)

        ii['action'] = 'find'
        ii['out'] = ''
        # Avoid recursion - use internal CM add function to add the script
        # artifact
        ii['common'] = True

        r = self.cmind.access(ii)
        if r['return'] > 0:
            return r

        lst = r['list']

        if len(lst) == 0:
            ii['action'] = 'add'
            ii['meta'] = {}

            # Tags must be unique for default
            r = self.cmind.access(ii)
            if r['return'] > 0:
                return r

            path = r['path']
        elif len(lst) > 1:
            return {
                'return': 1, 'error': 'ambiguity in cfg name - more than 1 CM artifact found'}
        else:
            path = lst[0].path

        # Check if has config
        path_to_cfg = os.path.join(path, 'config.json')

        config = {}
        if os.path.isfile(path_to_cfg):
            r = utils.load_json(path_to_cfg)
            if r['return'] > 0:
                return r

            config = r['meta']

        return {'return': 0, 'path': path,
                'path_to_config': path_to_cfg, 'config': config}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']
    env = i['env']

    if env.get('CM_GPTJ_INTEL_MODEL', '') == 'yes':
        i['run_script_input']['script_name'] = 'run-intel'
        harness_root = os.path.join(
            env['CM_MLPERF_INFERENCE_RESULTS_PATH'],
            'closed',
            'Intel',
            'code',
            'gptj-99',
            'pytorch-cpu')
        print(f"Harness Root: {harness_root}")
        env['CM_HARNESS_CODE_ROOT'] = harness_root
        env['CM_CALIBRATION_CODE_ROOT'] = os.path.join(
            env['CM_MLPERF_INFERENCE_RESULTS_PATH'], 'closed', 'Intel', 'calibration')

        env['CHECKPOINT_DIR'] = env['GPTJ_CHECKPOINT_PATH']

        env['QUANTIZED_MODEL_DIR'] = os.getcwd()

        if env['CM_ML_MODEL_WEIGHT_DATA_TYPES'] == "int8":
            env['INT8_MODEL_DIR'] = os.getcwd()
        else:
            env['INT4_MODEL_DIR'] = os.getcwd()

    elif env.get('CM_TMP_ML_MODEL_PROVIDER', '') == 'nvidia':
        i['run_script_input']['script_name'] = 'run-nvidia'
        if str(env.get('CM_DOCKER_DETACHED_MODE', '')
               ).lower() in ['yes', 'true', "1"]:
            env['DOCKER_RUN_OPTS'] = "--rm --ipc=host --ulimit memlock=-1 --ulimit stack=67108864"
        gpu_arch = int(
            float(
                env['CM_CUDA_DEVICE_PROP_GPU_COMPUTE_CAPABILITY']) *
            10)
        env['CM_GPU_ARCH'] = gpu_arch
        env['CM_TMP_REQUIRE_DOWNLOAD'] = 'no'

    else:
        is_saxml = env.get('CM_TMP_MODEL_SAXML', '')
        if is_saxml == "fp32":
            i['run_script_input']['script_name'] = 'run-saxml'
        elif is_saxml == "int8":
            i['run_script_input']['script_name'] = 'run-saxml-quantized'
        else:
            path = env.get('GPTJ_CHECKPOINT_PATH', '').strip()

            if path == '' or not os.path.exists(path):
                env['CM_TMP_REQUIRE_DOWNLOAD'] = 'yes'

    return {'return': 0}


def postprocess(i):

    env = i['env']

    if os.path.exists(os.path.join(
            env['GPTJ_CHECKPOINT_PATH'], "checkpoint-final")):
        env['GPTJ_CHECKPOINT_PATH'] = os.path.join(
            env['GPTJ_CHECKPOINT_PATH'], "checkpoint-final")

    is_saxml = env.get('CM_TMP_MODEL_SAXML', '')
    if is_saxml == "fp32":
        if os.path.exists("pax_gptj_checkpoint"):
            env['GPTJ_SAXML_CHECKPOINT_PATH'] = os.path.join(
                os.getcwd(), "pax_gptj_checkpoint")
            env['CM_ML_MODEL_FILE_WITH_PATH'] = env['GPTJ_SAXML_CHECKPOINT_PATH']
        else:
            return {'return': 1, 'error': 'pax_gptj_checkpoint generation failed'}

    elif is_saxml == "int8":
        if os.path.exists("int8_ckpt"):
            env['GPTJ_SAXML_INT8_CHECKPOINT_PATH'] = os.path.join(
                os.getcwd(), "int8_ckpt")
            env['CM_ML_MODEL_FILE_WITH_PATH'] = env['GPTJ_SAXML_INT8_CHECKPOINT_PATH']
        else:
            return {'return': 1, 'error': 'pax_gptj_checkpoint generation failed'}
    elif env.get('CM_TMP_ML_MODEL_PROVIDER', '') == 'nvidia':
        env['CM_ML_MODEL_FILE_WITH_PATH'] = os.path.join(
            env['CM_NVIDIA_MLPERF_SCRATCH_PATH'],
            'models',
            'GPTJ-6B',
            'fp8-quantized-ammo',
            'GPTJ-FP8-quantized')
    else:
        env['CM_ML_MODEL_FILE_WITH_PATH'] = env['GPTJ_CHECKPOINT_PATH']

    env['CM_ML_MODEL_FILE'] = os.path.basename(
        env['CM_ML_MODEL_FILE_WITH_PATH'])
    env['CM_GET_DEPENDENT_CACHED_PATH'] = env['CM_ML_MODEL_FILE_WITH_PATH']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import json
import shutil
import subprocess


def preprocess(i):

    os_info = i['os_info']
    env = i['env']
    state = i['state']
    script_path = i['run_script_input']['path']

    if env.get('CM_MLPERF_SKIP_RUN', '') == "yes":
        return {'return': 0}

    if env.get('CM_RUN_DOCKER_CONTAINER', '') == "yes":
        return {'return': 0}

    if env.get('CM_MLPERF_POWER', '') == "yes":
        power = "yes"
    else:
        power = "no"

    rerun = True if env.get("CM_RERUN", "") != '' else False

    if 'CM_MLPERF_MODEL' not in env:
        return {
            'return': 1, 'error': "Please select a variation specifying the model to run"}

    if 'CM_NUM_THREADS' not in env:
        if 'CM_MINIMIZE_THREADS' in env:
            env['CM_NUM_THREADS'] = str(int(env['CM_HOST_CPU_TOTAL_CORES']) //
                                        (int(env.get('CM_HOST_CPU_SOCKETS', '1')) * int(env.get('CM_HOST_CPU_TOTAL_CORES', '1'))))
        else:
            env['CM_NUM_THREADS'] = env.get('CM_HOST_CPU_TOTAL_CORES', '1')

    print("Using MLCommons Training source from '" +
          env['CM_MLPERF_TRAINING_SOURCE'] + "'")

    NUM_THREADS = env['CM_NUM_THREADS']

    if "bert" in env['CM_MLPERF_MODEL']:
        env['CM_RUN_DIR'] = os.path.join(
            env['CM_MLPERF_TRAINING_SOURCE'],
            "language_model",
            "tensorflow",
            "bert")

    return {'return': 0}


def postprocess(i):

    env = i['env']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    if os_info['platform'] == "windows":
        return {'return': 1, 'error': 'Script not supported in windows yet!'}

    print("Using MLCommons Inference source from '" +
          env['CM_MLPERF_INFERENCE_SOURCE'] + "'")

    # run cmd
    run_cmd = ""
    graph_folder = os.path.join(
        env['CM_MLPERF_INFERENCE_SOURCE'], 'graph', 'R-GAT')

    download_loc = env.get('CM_IGBH_DATASET_OUT_PATH', os.getcwd())

    env['CM_IGBH_DATASET_DOWNLOAD_LOCATION'] = download_loc

    run_cmd += f"cd {graph_folder} "
    x_sep = " && "

    # download the model
    if env['CM_IGBH_DATASET_TYPE'] == "debug":
        run_cmd += x_sep + env['CM_PYTHON_BIN_WITH_PATH'] + \
            f" tools/download_igbh_test.py --target-path {download_loc} "

    # split seeds
    run_cmd += x_sep + \
        f"{env['CM_PYTHON_BIN_WITH_PATH']} tools/split_seeds.py --path {download_loc} --dataset_size {env['CM_IGBH_DATASET_SIZE']}"

    # compress graph(for glt implementation)
    if env.get('CM_IGBH_GRAPH_COMPRESS', '') == "yes":
        run_cmd += x_sep + \
            f"{env['CM_PYTHON_BIN_WITH_PATH']} tools/compress_graph.py --path {download_loc} --dataset_size {env['CM_IGBH_DATASET_SIZE']} --layout {env['CM_IGBH_GRAPH_COMPRESS_LAYOUT']}"

    env['CM_RUN_CMD'] = run_cmd

    return {'return': 0}


def postprocess(i):

    env = i['env']

    env['CM_IGBH_DATASET_PATH'] = env.get(
        'CM_IGBH_DATASET_OUT_PATH', os.getcwd())

    print(
        f"Path to the IGBH dataset: {os.path.join(env['CM_IGBH_DATASET_PATH'], env['CM_IGBH_DATASET_SIZE'])}")

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    automation = i['automation']

    recursion_spaces = i['recursion_spaces']

    need_version = env.get('CM_VERSION', '')
    if need_version == '':
        return {'return': 1,
                'error': 'internal problem - CM_VERSION is not defined in env'}

    print(recursion_spaces + '    # Requested version: {}'.format(need_version))

#    if 'CM_GIT_CHECKOUT' not in env:
#        env['CM_GIT_CHECKOUT'] = 'releases/gcc-' + need_version

    if os_info['platform'] == 'windows':
        prefix = ''
        xos = 'windows'
        platform = 'x86_64'
        ext = '.exe'
    else:
        prefix = 'installer-'
        xos = env['CM_HOST_OS_TYPE']
        platform = env['CM_HOST_PLATFORM_FLAVOR']
        ext = '.sh'

    filename = 'bazel-{}-{}{}-{}{}'.format(need_version,
                                           prefix,
                                           xos,
                                           platform,
                                           ext)

    url = 'https://github.com/bazelbuild/bazel/releases/download/{}/{}'.format(
        need_version, filename)

    cur_dir = os.getcwd()

    if os_info['platform'] == 'windows':
        bazel_bin = 'bazel.exe'
        path = cur_dir
    else:
        bazel_bin = 'bazel'
        path = os.path.join(cur_dir, 'install', 'bin')

    env['CM_BAZEL_DOWNLOAD_URL'] = url
    env['CM_BAZEL_DOWNLOAD_FILE'] = filename

    env['CM_BAZEL_INSTALLED_PATH'] = path
    env['CM_BAZEL_BIN_WITH_PATH'] = os.path.join(path, bazel_bin)

    env['CM_GET_DEPENDENT_CACHED_PATH'] = os.getcwd()

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import json


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    if str(env.get('CUDA_SKIP_SUDO', '')).lower() == 'true':
        env['CM_SUDO'] = ''

    recursion_spaces = i['recursion_spaces']

    if os_info['platform'] == 'windows':
        file_name = env['CM_TMP_FILE_TO_CHECK_WINDOWS']

        if env.get('CM_INPUT', '').strip() == '' and env.get(
                'CM_TMP_PATH', '').strip() == '':
            # Check in "C:\Program Files\NVIDIA GPU Computing Toolkit\CUDA"
            paths = []
            for path in ["C:\\Program Files\\NVIDIA GPU Computing Toolkit\\CUDA",
                         "C:\\Program Files (x86)\\NVIDIA GPU Computing Toolkit\\CUDA"]:
                if os.path.isdir(path):
                    dirs = os.listdir(path)
                    for dr in dirs:
                        path2 = os.path.join(path, dr, 'bin')
                        if os.path.isdir(path2):
                            paths.append(path2)

            if len(paths) > 0:
                tmp_paths = ';'.join(paths)
                tmp_paths += ';' + os.environ.get('PATH', '')

                env['CM_TMP_PATH'] = tmp_paths
                env['CM_TMP_PATH_IGNORE_NON_EXISTANT'] = 'yes'

    else:
        file_name = env['CM_TMP_FILE_TO_CHECK_UNIX']

        # paths to cuda are not always in PATH - add a few typical locations to search for
        # (unless forced by a user)

        if env.get('CM_INPUT', '').strip() == '' and env.get(
                'CM_TMP_PATH', '').strip() == '':
            system_path = os.environ.get('PATH')
            if system_path:
                system_path = system_path + ":"
            env['CM_TMP_PATH'] = system_path + \
                '/usr/local/cuda/bin:/usr/cuda/bin:/usr/local/cuda-11/bin:/usr/cuda-11/bin:/usr/local/cuda-12/bin:/usr/cuda-12/bin:/usr/local/packages/cuda'
            env['CM_TMP_PATH_IGNORE_NON_EXISTANT'] = 'yes'

    if env['CM_CUDA_FULL_TOOLKIT_INSTALL'] == "yes":
        env_key = 'CM_NVCC_BIN_WITH_PATH'
        path_env_key = 'PATH'
    else:
        env_key = 'CM_CUDA_RT_WITH_PATH'
        path_env_key = 'LD_LIBRARY_PATH'
    env['CM_TMP_ENV_KEY'] = env_key

    if env_key not in env:
        r = i['automation'].find_artifact({'file_name': file_name,
                                           'env': env,
                                           'os_info': os_info,
                                           'default_path_env_key': path_env_key,
                                           'detect_version': True,
                                           'env_path_key': env_key,
                                           'run_script_input': i['run_script_input'],
                                           'recursion_spaces': recursion_spaces})
        if r['return'] > 0:
            if os_info['platform'] == 'windows':
                return r

            if r['return'] == 16 and env['CM_CUDA_FULL_TOOLKIT_INSTALL'] == "yes":
                env['CM_REQUIRE_INSTALL'] = "yes"
                return {'return': 0}
            else:
                return r

    return {'return': 0}


def detect_version(i):
    env = i['env']
    if env['CM_CUDA_FULL_TOOLKIT_INSTALL'] == "yes":
        return detect_version_nvcc(i)
    else:
        return detect_version_cuda_lib(i)


def detect_version_nvcc(i):
    r = i['automation'].parse_version({'match_text': r'release\s*([\d.]+)',
                                       'group_number': 1,
                                       'env_key': 'CM_CUDA_VERSION',
                                       'which_env': i['env']})
    if r['return'] > 0:
        return r

    version = r['version']

    print(i['recursion_spaces'] + '    Detected version: {}'.format(version))

    return {'return': 0, 'version': version}


def detect_version_cuda_lib(i):

    env = i['env']
    print(env)
    cuda_rt_file_path = env['CM_CUDA_RT_WITH_PATH']
    cuda_lib_path = os.path.dirname(cuda_rt_file_path)
    cuda_path = os.path.abspath(os.path.join(cuda_lib_path, os.pardir))

    cuda_version = "version-missing"

    version_json = os.path.join(cuda_path, "version.json")
    if os.path.exists(version_json):
        with open(version_json) as f:
            version_info = json.load(f)
            cuda_version_info = version_info.get('cuda_cudart')
            if cuda_version_info:
                cuda_version = cuda_version_info.get('version')

    env['CM_CUDA_VERSION'] = cuda_version
    version = cuda_version

    print(i['recursion_spaces'] + '    Detected version: {}'.format(version))

    return {'return': 0, 'version': version}


def postprocess(i):

    os_info = i['os_info']

    env = i['env']

    r = detect_version(i)
    if r['return'] > 0:
        return r
    version = r['version']

    env['CM_CUDA_CACHE_TAGS'] = 'version-' + version

    found_file_path = env[env['CM_TMP_ENV_KEY']]

    if env['CM_CUDA_FULL_TOOLKIT_INSTALL'] == "yes":

        cuda_path_bin = os.path.dirname(found_file_path)
        env['CM_CUDA_PATH_BIN'] = cuda_path_bin

        cuda_path = os.path.dirname(cuda_path_bin)
        env['CM_CUDA_INSTALLED_PATH'] = cuda_path
        env['CM_NVCC_BIN'] = os.path.basename(found_file_path)

    else:
        # We traverse backwards until we find a path with include dir
        parent_path = os.path.dirname(found_file_path)
        env['CM_CUDA_PATH_LIB'] = parent_path
        parent_path = os.path.dirname(parent_path)
        while os.path.isdir(parent_path):
            if os.path.exists(os.path.join(parent_path, "include")):
                print("Path is " + parent_path)
                found_path = parent_path
                cuda_path = found_path
                env['CM_CUDA_INSTALLED_PATH'] = cuda_path
                break
            else:
                parent_path = os.path.dirname(parent_path)

    if 'CM_CUDA_INSTALLED_PATH' not in env:
        return {
            'return': 1, 'error': "No CUDA installation path with an include directory is found"}

    env['CUDA_HOME'] = cuda_path
    env['CUDA_PATH'] = cuda_path

    cuda_system_path_install = False
    system_path = os.environ.get('PATH')
    if os.path.join(cuda_path, "bin") in system_path.split(":"):
        cuda_system_path_install = True

    # Check extra paths
    for key in ['+C_INCLUDE_PATH', '+CPLUS_INCLUDE_PATH',
                '+LD_LIBRARY_PATH', '+DYLD_FALLBACK_LIBRARY_PATH']:
        env[key] = []

    # Include
    cuda_path_include = os.path.join(cuda_path, 'include')
    if os.path.isdir(cuda_path_include):
        if os_info['platform'] != 'windows' and not cuda_system_path_install:
            env['+C_INCLUDE_PATH'].append(cuda_path_include)
            env['+CPLUS_INCLUDE_PATH'].append(cuda_path_include)

        env['CM_CUDA_PATH_INCLUDE'] = cuda_path_include

    # Lib
    if os_info['platform'] == 'windows':
        extra_dir = 'x64'
    else:
        extra_dir = ''

    for d in ['lib64', 'lib']:
        cuda_path_lib = os.path.join(cuda_path, d)

        if extra_dir != '':
            cuda_path_lib = os.path.join(cuda_path_lib, extra_dir)

        if os.path.isdir(cuda_path_lib):
            if not cuda_system_path_install:
                env['+LD_LIBRARY_PATH'].append(cuda_path_lib)
                env['+DYLD_FALLBACK_LIBRARY_PATH'].append(cuda_path_lib)

            env['CM_CUDA_PATH_LIB'] = cuda_path_lib
            break

    if '+ LDFLAGS' not in env:
        env['+ LDFLAGS'] = []
    if 'CM_CUDA_PATH_LIB' in env and not cuda_system_path_install:
        x = env['CM_CUDA_PATH_LIB']
        if ' ' in x:
            x = '"' + x + '"'
        env['+ LDFLAGS'].append("-L" + x)

    env['CM_CUDA_VERSION_STRING'] = "cu" + \
        env['CM_CUDA_VERSION'].replace(".", "")
    env['CM_MLPERF_SUT_NAME_RUN_CONFIG_SUFFIX5'] = env['CM_CUDA_VERSION_STRING']

    return {'return': 0, 'version': version}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import subprocess


def preprocess(i):

    env = i['env']
    state = i['state']

    os_info = i['os_info']

    # Update env variables
    env['CM_HOST_OS_TYPE'] = os_info['platform']
    env['CM_HOST_OS_BITS'] = os_info['bits']
    env['CM_HOST_PYTHON_BITS'] = os_info['python_bits']

    # Update state (demo)
    # state['os_info'] = os_info

    return {'return': 0}


def postprocess(i):

    env = i['env']
    state = i['state']

    os_info = i['os_info']

    if os_info['platform'] != 'windows':
        if os_info['platform'] == 'linux':
            sys_cmd = "ld --verbose | grep SEARCH_DIR "
            result = subprocess.check_output(
                sys_cmd, shell=True).decode("utf-8")
            result = result.replace("SEARCH_DIR(\"=", "")
            result = result.replace("SEARCH_DIR(\"", "")
            result = result.replace("\")", "")
            result = result.replace(" ", "")
            result = result.replace("\n", "")
            dirs = result.split(';')
            lib_dir = []
            for _dir in dirs:
                if _dir != '' and _dir not in lib_dir:
                    lib_dir.append(_dir)
            env['+CM_HOST_OS_DEFAULT_LIBRARY_PATH'] = lib_dir

        r = utils.load_txt(file_name='tmp-run.out',
                           check_if_exists=True,
                           split=True)
        if r['return'] > 0:
            return r

        s = r['list']

        state['os_uname_machine'] = s[0]
        state['os_uname_all'] = s[1]

        env['CM_HOST_OS_MACHINE'] = state['os_uname_machine']

    else:
        env['CM_HOST_OS_PACKAGE_MANAGER'] = "choco"

    import platform

    env['CM_HOST_SYSTEM_NAME'] = platform.node()

    if 'CM_HOST_OS_PACKAGE_MANAGER' not in env:
        if env.get('CM_HOST_OS_FLAVOR', '') == "ubuntu" or \
           "debian" in env.get('CM_HOST_OS_FLAVOR_LIKE', '') or \
           env.get('CM_HOST_OS_FLAVOR', '') == "debian":
            env['CM_HOST_OS_PACKAGE_MANAGER'] = "apt"
        if env.get('CM_HOST_OS_FLAVOR', '') == "rhel" or \
                "rhel" in env.get('CM_HOST_OS_FLAVOR_LIKE', ''):
            env['CM_HOST_OS_PACKAGE_MANAGER'] = "dnf"
        if env.get('CM_HOST_OS_FLAVOR', '') == "amzn":
            env['CM_HOST_OS_PACKAGE_MANAGER'] = "yum"
        if env.get('CM_HOST_OS_FLAVOR_LIKE', '') == "arch":
            env['CM_HOST_OS_PACKAGE_MANAGER'] = "arch"
        if env.get('CM_HOST_OS_FLAVOR', '') == "macos":
            env['CM_HOST_OS_PACKAGE_MANAGER'] = "brew"
        if env.get('CM_HOST_OS_FLAVOR', '') == "sles":
            env['CM_HOST_OS_PACKAGE_MANAGER'] = "zypper"
    if env.get('CM_HOST_OS_PACKAGE_MANAGER', '') == "apt":
        env['CM_HOST_OS_PACKAGE_MANAGER_INSTALL_CMD'] = "DEBIAN_FRONTEND=noninteractive apt-get install -y"
        env['CM_HOST_OS_PACKAGE_MANAGER_UPDATE_CMD'] = "apt-get update -y"
    elif env.get('CM_HOST_OS_PACKAGE_MANAGER', '') == "dnf":
        env['CM_HOST_OS_PACKAGE_MANAGER_INSTALL_CMD'] = "dnf install -y"
        env['CM_HOST_OS_PACKAGE_MANAGER_UPDATE_CMD'] = "dnf update -y"
    elif env.get('CM_HOST_OS_PACKAGE_MANAGER', '') == "pacman":
        env['CM_HOST_OS_PACKAGE_MANAGER_INSTALL_CMD'] = "pacman -Sy --noconfirm"
        env['CM_HOST_OS_PACKAGE_MANAGER_UPDATE_CMD'] = "pacman -Syu"
    elif env.get('CM_HOST_OS_PACKAGE_MANAGER', '') == "brew":
        env['CM_HOST_OS_PACKAGE_MANAGER_INSTALL_CMD'] = "brew install"
        env['CM_HOST_OS_PACKAGE_MANAGER_UPDATE_CMD'] = "brew update"
    elif env.get('CM_HOST_OS_PACKAGE_MANAGER', '') == "yum":
        env['CM_HOST_OS_PACKAGE_MANAGER_INSTALL_CMD'] = "yum install -y --skip-broken"
        env['CM_HOST_OS_PACKAGE_MANAGER_UPDATE_CMD'] = "yum update -y"
    elif env.get('CM_HOST_OS_PACKAGE_MANAGER', '') == "zypper":
        env['CM_HOST_OS_PACKAGE_MANAGER_INSTALL_CMD'] = "zypper install -y"
        env['CM_HOST_OS_PACKAGE_MANAGER_UPDATE_CMD'] = "zypper update -y"
    elif env.get('CM_HOST_OS_PACKAGE_MANAGER', '') == "choco":
        env['CM_HOST_OS_PACKAGE_MANAGER_INSTALL_CMD'] = "choco install -y"
        env['CM_HOST_OS_PACKAGE_MANAGER_UPDATE_CMD'] = "choco upgrade -y"

    if os.path.exists("/.dockerenv"):
        env['CM_RUN_INSIDE_DOCKER'] = "yes"

    return {'return': 0}



# Developer(s): Grigori Fursin

import streamlit as st
import os
import cmind

import misc


def page(i):

    st = i['st']
    params = i['params']
    script_path = i['script_path']
    script_alias = i['script_alias']
    script_tags = i['script_tags']
    skip_bottom = i.get('skip_bottom', False)

    extra = i.get('extra', {})

    meta = i['script_meta']
    repo_meta = i.get('script_repo_meta', None)

    no_run = os.environ.get('CM_GUI_NO_RUN', '')

    gui_meta = meta.get('gui', {})

    gui_func = gui_meta.get('use_customize_func', '')
    if gui_func != '':
        ii = {'streamlit_module': st,
              'meta': meta}
        return cmind.utils.call_internal_module(None, os.path.join(script_path, 'dummy'),
                                                'customize', gui_func, ii)

    st.markdown("""---""")

    if gui_meta.get('title', '') != '':
        title = gui_meta['title']

    # Set title
#    st.title('[Collective Mind](https://github.com/mlcommons/ck)')

    url_script = 'https://github.com/mlcommons/ck'
    if repo_meta is not None and script_alias != '':
        url = repo_meta.get('url', '')
        if url == '' and repo_meta.get('git', False):
            url = 'https://github.com/' + repo_meta['alias'].replace('@', '/')

        if url != '':
            # Recreate GitHub path
            if not url.endswith('/'):
                url = url + '/'

            url += 'tree/master/'

            if repo_meta.get('prefix', '') != '':
                url += repo_meta['prefix']

            if not url.endswith('/'):
                url = url + '/'

            url += 'script/' + script_alias

            url_script = url

    hide = params.get('hide_script_customization', False)

    if script_alias != '':
        show_customize = st.toggle(
    '**Customize input for the CM script "[{}]({})"**'.format(
        script_alias, url_script), value=not hide)
        hide = not show_customize

    # Check if found path and there is meta
    # TBD (Grigori): need to cache it using @st.cache
    variation_groups = {}
    default_variations = []
    variation_md = {}
    variation_alias = {}

    st_inputs = {}

    st_variations = {}

    if len(meta) > 0:
        variations = meta.get('variations', {})

        default_variation = meta.get('default_variation', '')

        variation_keys = sorted(list(variations.keys()))

        for variation_key in sorted(variation_keys):
            variation = variations[variation_key]

            alias = variation.get('alias', '').strip()

            if alias != '':
                aliases = variation_alias.get(alias, [])
                if variation_key not in aliases:
                    aliases.append(variation_key)
                variation_alias[alias] = aliases

                # Do not continue this loop if alias
                continue

            if 'default_gui' in variation:
                default = variation['default_gui']
            else:
                default = variation.get('default', False)

            if not default:
                # Check outdated
                if default_variation == variation_key:
                    default = True

            extra1 = ''
            extra2 = ''
            if default:
                extra1 = '**'
                extra2 = '** (default)'

                default_variations.append(variation_key)

            group = variation.get('group', '')

            if variation_key.endswith('_'):
                group = '*internal*'
            elif group == '':
                group = '*no-group*'

            if group not in variation_groups:
                variation_groups[group] = []

            variation_groups[group].append(variation_key)

        # Prepare variation_groups
        if len(variations) > 0:
            if not hide:
                st.markdown(
                    '**Select variations to update multiple flags and environment variables:**')

            variation_groups_order = meta.get('variation_groups_order', [])
            for variation in sorted(variation_groups):
                if variation not in variation_groups_order:
                    variation_groups_order.append(variation)

            for group_key in variation_groups_order:
                group_key_cap = group_key.replace('-', ' ').capitalize()
                if not group_key.startswith('*'):
                    y = ['']

                    index = 0
                    selected_index = 0
                    for variation_key in sorted(variation_groups[group_key]):
                        index += 1
                        y.append(variation_key)
                        if variation_key in default_variations:
                            selected_index = index

                    key2 = '~~' + group_key

                    x = params.get(key2, None)
                    if x != None and len(x) > 0 and x[0] != None:
                        x = x[0]
                        if x in y:
                            selected_index = y.index(x) if x in y else 0

                    if hide:
                        st_variations[key2] = sorted(y)[selected_index]
                    else:
                        st_variations[key2] = st.selectbox(
                            group_key_cap, sorted(y), index=selected_index, key=key2)

                elif group_key == '*no-group*':
                    for variation_key in sorted(variation_groups[group_key]):
                        v = False
                        if variation_key in default_variations:
                            v = True

                        key2 = '~' + variation_key

                        x = params.get(key2, None)
                        if x != None and len(x) > 0 and x[0] != None:
                            if x[0].lower() == 'true':
                                v = True
                            elif x[0].lower() == 'false':
                                v = False

                        if hide:
                            st_variations[key2] = v
                        else:
                            st_variations[key2] = st.checkbox(
                                variation_key.capitalize(), key=key2, value=v)

        # Prepare inputs
        input_desc = meta.get('input_description', {})

        if len(input_desc) > 0:

            sort_desc = {}
            sort_keys = []
            for k in input_desc:
                sort = input_desc[k].get('sort', 0)
                if sort > 0:
                    sort_desc[k] = sort
            if len(sort_desc) > 0:
                sort_keys = sorted(sort_desc, key=lambda k: sort_desc[k])

            other_keys = sorted(
                [k for k in input_desc if input_desc[k].get('sort', 0) == 0])

            all_keys = [] if len(sort_keys) == 0 else sort_keys
            all_keys += other_keys

            if not hide:
                if len(sort_keys) > 0:
                    st.markdown('**Select main flags:**')
                else:
                    st.markdown('**Select all flags:**')

            other_flags = False
            for key in all_keys:
                value = input_desc[key]

                if len(sort_keys) > 0 and value.get(
                    'sort', 0) == 0 and not other_flags:
                    if not hide:
                        st.markdown('**Select other flags:**')
                    other_flags = True

                ii = {'key': key,
                      'desc': value,
                      'params': params,
                      'st': st,
                      'st_inputs': st_inputs,
                      'hide': hide}

                r2 = misc.make_selector(ii)
                if r2['return'] > 0:
                    return r2

    # Check tags
    selected_variations = []
    for k in st_variations:
        v = st_variations[k]

        if k.startswith('~~'):
            k2 = k[2:]
        elif k.startswith('~'):
            k2 = k[1:]


if isinstance(v,         if )            if v:
                selected_variations.append('_' + k2)
        elif v != '':
            selected_variations.append('_' + v)

    x = script_tags
    if ' ' in script_tags:
        if len(selected_variations) > 0:
            x += ' ' + ' '.join(selected_variations)

        tags = '"{}"'.format(x)
    else:
        if len(selected_variations) > 0:
            x += ',' + ','.join(selected_variations)

        tags = '--tags={}'.format(x)

    # Add extras to inputs
    add_to_st_inputs = extra.get('add_to_st_inputs', {})
    if len(add_to_st_inputs) > 0:
        st_inputs.update(add_to_st_inputs)

    ##########################################################################
    st.markdown("""---""")
    st.markdown('**Run this CM script (Linux/MacOS/Windows):**')

    x = ''

    extra_notes_online = extra.get('extra_notes_online', '')
    if extra_notes_online != '':
        x += ' [ ' +extra_notes_online +' ] '

    extra_faq_online = extra.get('extra_faq_online', '')
    if extra_faq_online != '':
        x += ' [ ' +extra_faq_online +' ] '

    if x != '':
        st.markdown('*' + x.strip() + '*')

    host_os_windows = False if os.name != 'nt' else True
    host_os_use_windows = st.toggle('Run on Windows?', value=host_os_windows)
    if host_os_use_windows:
        var1 = '^'
        host_os_flag = 'windows'
# st.markdown('*Check how to install [a few
# dependencies](https://github.com/mlcommons/ck/blob/master/docs/installation.md#windows)
# on Windows.*')
    else:
        var1 = '\\'
        host_os_flag = 'linux'

    show_cm_install = st.toggle(
        'Install MLCommons Collective Mind',
        value=False)

    if show_cm_install:

        import playground_install
        extra = {'skip_header': True,
                 'run_on_windows': host_os_use_windows}
        r = playground_install.page(st, params, extra)
        if r['return'] > 0:
            return r

        st.markdown('---')

    ##########################################################################
    shell = st.toggle('Open shell after executing CM script?', value=False)
    if shell:
        st_inputs['~shell'] = True

    ##########################################################################
    flags_dict = {}
    flags = ''

    for key in st_inputs:
        value = st_inputs[key]
        key2 = key[1:]

        if value != '' and (type(value) !=bool or value==True):
            flags += ' ' + var1 + '\n   --' + key2

            z = True
if not isinstance(value,             if )                x = str(value)
                z = x

                if ' ' in x or ':' in x or '/' in x or '\\' in x:
                    x = '"' + x + '"'
                flags += '=' + x

            flags_dict[key2] = z

    ##########################################################################
    run_via_docker = False
    if not extra.get('skip_script_docker_func', False) and len(meta.get('docker', {})) > 0:
        run_via_docker = st.toggle(
            'Use Docker',
            key='run_via_docker',
            value=False)

        if run_via_docker:
            st.markdown(
                "*WARNING: CM automatically generates containers for a give script - it's a beta functionality - feel free to [test and provide feedback](https://discord.gg/JjWNWXKxwT)!*")

    action = 'docker' if run_via_docker else 'run'
    cli = 'cm {} script {} {}\n'.format(action, tags, flags)

    ##########################################################################
    use_experiment_from_extra = extra.get('use_experiment', False)

    use_experiment = st.toggle(
        'Use CM experiment for reproducibility',
        key='use_cm_experiment',
        value=use_experiment_from_extra)

    extra_cm_prefix = ''
    if use_experiment:
        cli = 'cm run experiment --tags={} -- {}\n '.format(
            "repro," + script_tags, var1) + cli

    ##########################################################################

    extra_setup = extra.get('extra_setup', '').strip()
    if len(extra_setup) > 2:
        show_extra_setup_notes = st.toggle(
    'Show extra setup notes?', value=True)

        if show_extra_setup_notes:
            #            st.markdown('---')
            st.markdown(extra_setup)
#            st.markdown('---')

    show_python_api = st.toggle('Run via Python API', value=False)

    # Python API
    if show_python_api:

        final_script_tags = script_tags
        if len(selected_variations) > 0:
            for sv in selected_variations:
                final_script_tags += ' ' + sv
        final_script_tags = final_script_tags.replace(' ', ',')

        if use_experiment:
            dd = {
                'action': 'run',
                'automation': 'experiment,a0a2d123ef064bcb',
                'tags': script_tags,
                  'out': 'con'
            }

            unparsed_cmd = ['cm',
                            'run',
                            'script,5b4e0237da074764',
                            '--tags=' + final_script_tags]

            for flag in flags_dict:
                value = flags_dict[flag]
                unparsed_cmd.append('--' + flag + '=' + str(value))

            dd['unparsed_cmd'] = unparsed_cmd

        else:
            dd = {
                'action': action,
                'automation': 'script,5b4e0237da074764',
            }

            dd['tags'] = final_script_tags

            dd['out'] = 'con'

            dd.update(flags_dict)

        import json
        dd_json = json.dumps(dd, indent=2)
        dd_json = dd_json.replace(
            ': true',
            ': True').replace(
        ': false',
            ': False')

        y = 'import cmind\n'
        y += 'r = cmind.access(' + dd_json + ')\n'
        y += 'if r[\'return\']>0: print (r[\'error\'])\n'

        x = '''
    ```python
              {}
          '''.format(y)

    #    st.write(x.replace('\n','<br>\n'), unsafe_allow_html=True)

        st.markdown(x)

    ##########################################################################
    show_cli = st.toggle('Run from the command line', value=True)

    if show_cli:
        # Add explicit button "Run"
        cli = st.text_area('', cli, height=600)

        if no_run == '' and st.button("Run in the new terminal"):
            cli = cli + var1 + '--pause\n'

            cli = cli.replace(var1, ' ').replace('\n', ' ')

            if os.name == 'nt':
                cmd2 = 'start cmd /c {}'.format(cli)
            else:
                cli2 = cli.replace('"', '\\"')

                prefix = os.environ.get('CM_GUI_SCRIPT_PREFIX_LINUX', '')
                if prefix != '':
                    prefix += ' '

                cmd2 = prefix + 'bash -c "{}"'.format(cli2)

            print('Running command:')
            print('')
            print('  {}'.format(cmd2))
            print('')

            os.system(cmd2)

    # Some info
    x = '''
         <i>
         <center>
         We would like to thank <a href="https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md">all Collective Mind users and contributors</a>
         for supporting this collaborative engineering effort -<br>
         please don't hesitate report issues or suggest features at <a href="https://github.com/mlcommons/ck/issues">CM GitHub</a>!
         </center>
        '''
    st.write(x, unsafe_allow_html=True)

    return {'return': 0}


if __name__ == "__main__":
    main()



import os
from pip._internal.operations import freeze

pip_freeze_out = os.environ.get(
    'CM_DUMP_RAW_PIP_FREEZE_FILE_PATH',
    'tmp-pip-freeze')

if os.path.isfile(pip_freeze_out):
    os.remove(pip_freeze_out)

pkgs = freeze.freeze()

x = ''

try:
    for pkg in pkgs:
        x += pkg + '\n'
except BaseException:
    pass

if len(x) > 0:
    with open(pip_freeze_out, "w") as f:
        f.write(x)



import pycuda.driver as cuda
import pycuda.autoinit


def get_gpu_info():
    num_gpus = cuda.Device.count()
    all_gpu_info = []

    for i in range(num_gpus):
        device = cuda.Device(i)
        cuda_runtime_version = cuda.get_version()
        cuda_runtime_version_str = f"{cuda_runtime_version[0]}.{cuda_runtime_version[1]}"

        gpu_info = {
            "GPU Device ID": device.pci_bus_id(),
            "GPU Name": device.name(),
            "GPU compute capability": f"{device.compute_capability()[0]}.{device.compute_capability()[1]}",
            "CUDA driver version": f"{cuda.get_driver_version() // 1000}.{(cuda.get_driver_version() % 1000) // 10}",
            "CUDA runtime version": cuda_runtime_version_str,
            "Global memory": device.total_memory(),
            "Max clock rate": f"{device.get_attribute(cuda.device_attribute.CLOCK_RATE)} MHz",
            "Total amount of shared memory per block": device.get_attribute(cuda.device_attribute.MAX_SHARED_MEMORY_PER_BLOCK),
            "Total number of registers available per block": device.get_attribute(cuda.device_attribute.MAX_REGISTERS_PER_BLOCK),
            "Warp size": device.get_attribute(cuda.device_attribute.WARP_SIZE),
            "Maximum number of threads per multiprocessor": device.get_attribute(cuda.device_attribute.MAX_THREADS_PER_MULTIPROCESSOR),
            "Maximum number of threads per block": device.get_attribute(cuda.device_attribute.MAX_THREADS_PER_BLOCK),
            "Max dimension size of a thread block X": device.get_attribute(cuda.device_attribute.MAX_BLOCK_DIM_X),
            "Max dimension size of a thread block Y": device.get_attribute(cuda.device_attribute.MAX_BLOCK_DIM_Y),
            "Max dimension size of a thread block Z": device.get_attribute(cuda.device_attribute.MAX_BLOCK_DIM_Z),
            "Max dimension size of a grid size X": device.get_attribute(cuda.device_attribute.MAX_GRID_DIM_X),
            "Max dimension size of a grid size Y": device.get_attribute(cuda.device_attribute.MAX_GRID_DIM_Y),
            "Max dimension size of a grid size Z": device.get_attribute(cuda.device_attribute.MAX_GRID_DIM_Z),
        }

        all_gpu_info.append(gpu_info)

    return all_gpu_info


# Print the GPU information for all available GPUs
if __name__ == "__main__":
    gpu_info_list = get_gpu_info()
    with open("tmp-run.out", "w") as f:
        for idx, gpu_info in enumerate(gpu_info_list):
            print(f"GPU {idx}:")
            for key, value in gpu_info.items():
                f.write(f"{key}: {value}\n")



import argparse
import networkx as nx
import matplotlib.pyplot as plt
import json

# Function to parse the nested JSON structure


def parse_json_to_edges(json_data):
    edges = []
    for root_key, nodes in json_data.items():
        for node in nodes:
            for node_key, node_details in node.items():
                edges.append((node_details["parent"], node_key))
    return edges


def generate_mermaid_output(json_data, mermaid_file="graph.mmd"):
    edges = parse_json_to_edges(json_data)

    # Start the Mermaid graph definition
    mermaid_lines = ["graph TD"]  # Use "TD" for top-down; "LR" for left-right

    # Add each edge in Mermaid syntax
    for parent, child in edges:
        mermaid_lines.append(
            f"""    {parent.replace(" ", "_")} --> {child.replace(" ", "_")}""")

    # Write to a Mermaid file
    with open(mermaid_file, "w") as f:
        f.write("\n".join(mermaid_lines))

    print(f"Mermaid syntax saved to {mermaid_file}")


# Function to generate and visualize the graph
def generate_graph_from_nested_json(json_data, output_image="graph.png"):
    # Parse the JSON to extract edges
    edges = parse_json_to_edges(json_data)

    # Initialize a directed graph
    G = nx.DiGraph()

    # Add edges to the graph
    G.add_edges_from(edges)

    # Draw the graph using a spring layout for better visualization
    plt.figure(figsize=(30, 25))
    # pos = nx.spectral_layout(G, seed=42)  # Seed for consistent layout
    pos = nx.shell_layout(G)  # Seed for consistent layout
    nx.draw(
        G,
        pos,
        with_labels=True,
        node_size=7000,
        node_color="skyblue",
        font_size=9,
        font_color="darkblue",
        edge_color="gray",
        arrowsize=20,
        linewidths=1.5
    )
    plt.title("Parent-Child Graph from Nested JSON", fontsize=16)

    # Save the visualization
    plt.savefig(output_image, format="png", dpi=300)
    print(f"Graph visualization saved as {output_image}")
    # plt.show()

    return G

# Function to export the graph data


def export_graph_data(graph, filename="graph.graphml"):
    nx.write_graphml(graph, filename)
    print(f"Graph data saved as {filename}")

# Main function to handle argument parsing and processing


def main():
    parser = argparse.ArgumentParser(
        description="Generate a graph from nested JSON input.")
    parser.add_argument(
        "json_file",
        type=str,
        help="Path to the JSON input file.")
    parser.add_argument(
        "--output_image",
        type=str,
        default="graph.png",
        help="Output image file for the graph visualization.")
    parser.add_argument(
        "--output_mermaid",
        type=str,
        default="graph.mmd",
        help="Output mermaid file for the graph data.")
    parser.add_argument(
        "--output_graphml",
        type=str,
        default="graph.graphml",
        help="Output GraphML file for the graph data.")

    args = parser.parse_args()

    # Load the JSON input file
    with open(args.json_file, "r") as f:
        json_data = json.load(f)

    # Generate the graph
    G = generate_graph_from_nested_json(
        json_data, output_image=args.output_image)

    generate_mermaid_output(json_data, mermaid_file=args.output_mermaid)

    # Export the graph data
    export_graph_data(G, filename=args.output_graphml)


if __name__ == "__main__":
    main()



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil


def preprocess(i):

    env = i['env']

    script_path = i['run_script_input']['path']

    if env.get('CM_GIT_PATCH_FILENAMES', '') != '':
        patch_files = env['CM_GIT_PATCH_FILENAMES'].split(",")
        patch_files_full_paths = []
        for patch_file in patch_files:
            patch_file_full_path = os.path.join(
                script_path, "patch", patch_file)
            patch_files_full_paths.append(patch_file_full_path)
        env['CM_GIT_PATCH_FILEPATHS'] = ",".join(patch_files_full_paths)

    return {'return': 0}


def postprocess(i):

    env = i['env']
    state = i['state']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    if os_info['platform'] == 'windows':
        return {'return': 1, 'error': 'Windows is not supported in this script yet'}

    env = i['env']

    if env.get('CM_MLPERF_INFERENCE_INTEL_LANGUAGE_MODEL', '') == "yes":
        i['run_script_input']['script_name'] = "run-intel-mlperf-inference-v3_1"
        run_cmd = "CC=clang CXX=clang++ USE_CUDA=OFF python -m pip install -e . "

        env['CM_RUN_CMD'] = run_cmd
    elif env.get('CM_MLPERF_INFERENCE_INTEL_MODEL', '') in ["resnet50", "retinanet"]:
        i['run_script_input']['script_name'] = "run-intel-mlperf-inference-vision"
        run_cmd = f"CC={env['CM_C_COMPILER_WITH_PATH']} CXX={env['CM_CXX_COMPILER_WITH_PATH']} USE_CUDA=OFF python -m pip install -e . "

        env['CM_RUN_CMD'] = run_cmd

    if not env.get('+ CFLAGS', []):
        env['+ CFLAGS'] = []
    if not env.get('+ CXXFLAGS', []):
        env['+ CXXFLAGS'] = []

    env['+ CFLAGS'] += ["-Wno-error=uninitialized",
                        "-Wno-error=maybe-uninitialized", "-fno-strict-aliasing"]
    env['+ CXXFLAGS'] += ["-Wno-error=uninitialized",
                          "-Wno-error=maybe-uninitialized", "-fno-strict-aliasing"]
    automation = i['automation']

    recursion_spaces = i['recursion_spaces']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):
    os_info = i['os_info']
    env = i['env']

    return {'return': 0}


def postprocess(i):

    os_info = i['os_info']
    env = i['env']

    env['CM_MLPERF_RUN_CMD'] = env.get('CM_RUN_CMD')

    if env.get('CM_MLPERF_POWER', '') == "yes":

        if env.get('CM_MLPERF_SHORT_RANGING_RUN', '') != 'no':
            # Write '0' to the count.txt file in CM_RUN_DIR
            count_file = os.path.join(env.get('CM_RUN_DIR', ''), 'count.txt')
            with open(count_file, 'w') as f:
                f.write('0')

            if os_info['platform'] != 'windows':
                # Construct the shell command with proper escaping
                env['CM_MLPERF_RUN_CMD'] = r"""
CM_MLPERF_RUN_COUNT=\$(cat \${CM_RUN_DIR}/count.txt);
echo \${CM_MLPERF_RUN_COUNT};
CM_MLPERF_RUN_COUNT=\$((CM_MLPERF_RUN_COUNT+1));
echo \${CM_MLPERF_RUN_COUNT} > \${CM_RUN_DIR}/count.txt;

if [ \${CM_MLPERF_RUN_COUNT} -eq 1 ]; then
export CM_MLPERF_USER_CONF="${CM_MLPERF_RANGING_USER_CONF}";
else
export CM_MLPERF_USER_CONF="${CM_MLPERF_TESTING_USER_CONF}";
fi
;

                """ + env.get('CM_RUN_CMD', '').strip()
            else:
                env['CM_MLPERF_RUN_CMD'] = r"""
:: Read the current count from the file
set /p CM_MLPERF_RUN_COUNT=<%CM_RUN_DIR%\count.txt
echo !CM_MLPERF_RUN_COUNT!

:: Increment the count
set /a CM_MLPERF_RUN_COUNT=!CM_MLPERF_RUN_COUNT! + 1
echo !CM_MLPERF_RUN_COUNT! > %CM_RUN_DIR%\count.txt

:: Check the value and set the environment variable accordingly
if !CM_MLPERF_RUN_COUNT! EQU 1 (
    set CM_MLPERF_USER_CONF=%CM_MLPERF_RANGING_USER_CONF%
) else (
    set CM_MLPERF_USER_CONF=%CM_MLPERF_TESTING_USER_CONF%
)
                """ + env.get('CM_RUN_CMD', '').strip()
        else:
            # Just use the existing CM_RUN_CMD if no ranging run is needed
            env['CM_MLPERF_RUN_CMD'] = env.get('CM_RUN_CMD', '').strip()

    return {'return': 0}



# Author: Grigori Fursin
# Contributors: Arjun Suresh, Anandhu Sooraj
#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

import os
from cmind import utils


############################################################
def copy_to_remote(i):
    """
    Add CM automation.

    Args:
      (CM input dict):

      (out) (str): if 'con', output to console

      parsed_artifact (list): prepared in CM CLI or CM access function
                                [ (artifact alias, artifact UID) ] or
                                [ (artifact alias, artifact UID), (artifact repo alias, artifact repo UID) ]

      (repos) (str): list of repositories to search for automations (internal & mlcommons@ck by default)

      (output_dir) (str): output directory (./ by default)

    Returns:
      (CM return dict):

      * return (int): return code == 0 if no error and >0 if error
      * (error) (str): error string if return>0

    """

    self_module = i['self_module']

    remote_host = i.get('remote_host')
    if not remote_host:
        return {'return': 1,
            'error': 'Please input remote host_name/IP via --remote_host'}
    remote_cm_repos_location = i.get(
    'remote_cm_repos_location', os.path.join(
        "/home", os.getlogin(), "CM", "repos"))
    remote_cm_cache_location = os.path.join(
    remote_cm_repos_location, "local", "cache")

    remote_port = i.get('remote_port', '22')
    remote_user = i.get('remote_user', os.getlogin())

    tag_string = i['tags']
    tag_string += ",-tmp"

    cm_input = {'action': 'show',
                'automation': 'cache',
                'tags': f'{tag_string}',
                'quiet': True
                }
    r = self_module.cmind.access(cm_input)
    if r['return'] > 0:
        return r

    if len(r['list']) == 0:
        pass  # fixme
    elif len(r['list']) > 1:
        print("Multiple cache entries found: ")
        for k in sorted(r['list'], key=lambda x: x.meta.get('alias', '')):
            print(k.path)
        x = input("Would you like to copy them all? Y/n: ")
        if x.lower() == 'n':
            return {'return': 0}

    import json

    for k in sorted(r['list'], key=lambda x: x.meta.get('alias', '')):
        path = k.path
        cacheid = os.path.basename(path)

        copy_cmd = f"rsync -avz --exclude cm-cached-state.json -e 'ssh -p {remote_port}' {path} {remote_user}@{remote_host}:{remote_cm_cache_location}"
        print(copy_cmd)
        os.system(copy_cmd)

        cm_cached_state_json_file = os.path.join(path, "cm-cached-state.json")
        if not os.path.exists(cm_cached_state_json_file):
            return {'return': 1,
                'error': f'cm-cached-state.json file missing in {path}'}

        with open(cm_cached_state_json_file, "r") as f:
            cm_cached_state = json.load(f)

        new_env = cm_cached_state['new_env']
        new_state = cm_cached_state['new_state']  # Todo fix new state
        cm_repos_path = os.environ.get(
    'CM_REPOS', os.path.join(
        os.path.expanduser("~"), "CM", "repos"))
        cm_cache_path = os.path.realpath(
            os.path.join(cm_repos_path, "local", "cache"))

        for key, val in new_env.items():


if isinstance(val,             if )                new_env[key] = val.replace(
    cm_cache_path, remote_cm_cache_location)

        with open("tmp_remote_cached_state.json", "w") as f:
            json.dump(cm_cached_state, f, indent=2)

        remote_cached_state_file_location = os.path.join(
    remote_cm_cache_location, cacheid, "cm-cached-state.json")
        copy_cmd = f"rsync -avz -e 'ssh -p {remote_port}' tmp_remote_cached_state.json {remote_user}@{remote_host}:{remote_cached_state_file_location}"
        print(copy_cmd)
        os.system(copy_cmd)

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil


def preprocess(i):

    env = i['env']

    print("Using MLCommons Inference source from '" +
          env['CM_MLPERF_INFERENCE_SOURCE'] + "'")
    preprocess_src = os.path.join(
        env['CM_MLPERF_INFERENCE_3DUNET_PATH'],
        'preprocess.py')
    cmd = 'cd ' + env['CM_MLPERF_INFERENCE_3DUNET_PATH'] + \
        ' && ${CM_PYTHON_BIN_WITH_PATH} preprocess.py --raw_data_dir ' + \
        env['CM_DATASET_PATH'] + ' --results_dir ' + \
        os.getcwd() + ' --mode preprocess'
    env['CM_TMP_CMD'] = cmd

    return {'return': 0}


def postprocess(i):
    env = i['env']
    if 'CM_DATASET_PREPROCESSED_PATH' not in env:
        env['CM_DATASET_PREPROCESSED_PATH'] = os.getcwd()
        env['CM_DATASET_KITS19_PREPROCESSED_PATH'] = env['CM_DATASET_PREPROCESSED_PATH']

    return {'return': 0}



# This test covers script installation, version, shared library install

import cmind as cm
import check as checks

r = cm.access({'action': 'run',
               'automation': 'script',
               'tags': 'python,src,install,_shared',
               'version': '3.9.10',
               'quiet': 'true'})
checks.check_return(r)

r = cm.access({'action': 'search', 'automation': 'cache',
              'tags': 'python,src,install,_shared,version-3.9.10'})
checks.check_list(r, "python,src,install,_shared,version-3.9.10")



"""
Testing CM debugging

# Developer(s): Grigori Fursin
"""

import cmind.utils
import os
import json

print("Hello World 1")

env = os.environ

print('')
print(json.dumps(dict(env), indent=2))

# Import cmind to test break points
if os.environ.get('CM_TMP_DEBUG_UID', '') == '45a7c3a500d24a63':
    cmind.utils.debug_here(
        __file__,
        port=5678,
        text='Debugging main.py!').breakpoint()

print('')
print("Hello World 2")



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil


def preprocess(i):

    os_info = i['os_info']

    if os_info['platform'] == 'windows':
        return {'return': 1, 'error': 'Windows is not supported in this script yet'}
    env = i['env']

    if '+LIBRARY_PATH' not in env:
        env['+LIBRARY_PATH'] = []

    if 'CM_TENSORRT_INSTALL_PATH' in env:
        env['+LIBRARY_PATH'].append(os.path.join(
            env['CM_TENSORRT_INSTALL_PATH'], "lib"))

    cxxflags = [
        "-Wno-error=switch",
        "-DDALI_1_15=1",
        "-Wno-error=maybe-uninitialized"]

    if env.get('CM_GCC_VERSION', '') != '':
        gcc_major_version = env['CM_GCC_VERSION'].split(".")[0]
        if int(gcc_major_version) > 10:
            if env.get('CM_MLPERF_INFERENCE_VERSION', '') != "4.1":
                cxxflags.append("-Wno-error=range-loop-construct")

    if env.get('CM_MLPERF_DEVICE', '') == "inferentia":
        env['USE_INFERENTIA'] = "1"
        env['USE_NIGHTLY'] = "0"
        env['CM_MAKE_BUILD_COMMAND'] = "build"

    if '+ CXXFLAGS' not in env:
        env['+ CXXFLAGS'] = []

    env['+ CXXFLAGS'] += cxxflags
    return {'return': 0}


def postprocess(i):

    env = i['env']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

# Developers: Grigori Fursin

from cmind import utils
import os


def preprocess(i):

    print('')
    print('customize.py: preprocess')
    print('')

    return {'return': 0}


def postprocess(i):

    automation = i['automation']
    run_script_input = i['run_script_input']
    env = i['env']

    print('')
    print('customize.py: postprocess')
    print('')

    r = automation.run_native_script(
        {'run_script_input': run_script_input, 'env': env, 'script_name': 'run2'})
    if r['return'] > 0:
        return r

    return {'return': 0}


def detect_version(i):

    print('')
    print('customize.py: detect_version')
    print('')

    return {'return': 0}



import sys
import os
import cmind as cm
import check as checks
import json
import yaml

files = sys.argv[1:]

for file in files:
    print(file)
    if not os.path.isfile(file) or not "script" in file:
        continue
    if not file.endswith("_cm.json") and not file.endswith("_cm.yaml"):
        continue
    script_path = os.path.dirname(file)
    f = open(file)
    if file.endswith(".json"):
        data = json.load(f)
    elif file.endswith(".yaml"):
        data = yaml.safe_load(f)
    if data.get('uid', '') == '':
        continue  # not a CM script meta
    uid = data['uid']

    ii = {
        'action': 'test', 'automation': 'script', 'artifact': uid, 'quiet': 'yes', 'out': 'con'
    }
    if os.environ.get('DOCKER_CM_REPO', '') != '':
        ii['docker_cm_repo'] = os.environ['DOCKER_CM_REPO']
    if os.environ.get('DOCKER_CM_REPO_BRANCH', '') != '':
        ii['docker_cm_repo_branch'] = os.environ['DOCKER_CM_REPO_BRANCH']
    if os.environ.get('TEST_INPUT_INDEX', '') != '':
        ii['test_input_index'] = os.environ['TEST_INPUT_INDEX']
    print(ii)
    r = cm.access(ii)

    checks.check_return(r)



# This test covers version, variation, compilation from src, add_deps,
# add_deps_recursive, deps, post_deps

import cmind as cm
from pathlib import Path
import sys
import os

sys.path.insert(
    1,
    os.path.join(
        Path(__file__).parent.parent.resolve(),
        "script"))
import check as checks  # noqa

r = cm.access({'action': 'run', 'automation': 'script', 'tags': 'app,mlperf,inference,generic,_cpp,_retinanet,_onnxruntime,_cpu', 'adr':
               {'python': {'version_min': '3.8'}, 'compiler': {'tags': "gcc"}, 'openimages-preprocessed': {'tags': '_50'}}, 'scenario': 'Offline',
               'mode': 'accuracy', 'test_query_count': '10', 'rerun': 'true', 'quiet': 'yes'})
checks.check_return(r)

r = cm.access({'action': 'run', 'automation': 'script', 'tags': 'app,mlperf,inference,generic,_cpp,_retinanet,_onnxruntime,_cpu', 'adr':
               {'python': {'version_min': '3.8'}, 'compiler': {'tags': "gcc"}, 'openimages-preprocessed': {'tags': '_50'}}, 'scenario': 'Offline',
               'mode': 'performance', 'test_query_count': '10', 'rerun': 'true', 'quiet': 'yes'})
checks.check_return(r)

r = cm.access({'action': 'run',
               'automation': 'script',
               'tags': 'install,python-venv',
               'version': '3.10.8',
               'name': 'mlperf'})
checks.check_return(r)

r = cm.access({'action': 'run', 'automation': 'script', 'tags': 'run,mlperf,inference,generate-run-cmds,_submission,_short', 'adr':
               {'python': {'name': 'mlperf', 'version_min': '3.8'}, 'compiler': {'tags': "gcc"}, 'openimages-preprocessed': {'tags': '_50'}}, 'submitter': 'Community',
               'implementation': 'cpp', 'hw_name': 'default', 'model': 'retinanet', 'backend': 'onnxruntime', 'device': 'cpu', 'scenario': 'Offline',
               'test_query_count': '10', 'clean': 'true', 'quiet': 'yes'})
checks.check_return(r)



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    meta = i['meta']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    automation = i['automation']

    recursion_spaces = i['recursion_spaces']

    # Add extra tags to python
    add_extra_cache_tags = []  # for this script
    add_python_extra_cache_tags = ['virtual']  # for get-python script

    name = env.get('CM_NAME', '')
    if not quiet and name == '':
        print('')
        x = input(
            'Enter some tag to describe this virtual env (mlperf-inf,octoml-bench,etc): ')
        x = x.strip()

        if x != '':
            name = x

    directory_name = 'venv'
    if name != '':
        directory_name = name.strip().lower()
        name_tag = 'name-' + directory_name

        add_extra_cache_tags.append(name_tag)
        add_python_extra_cache_tags.append(name_tag)

    env['CM_VIRTUAL_ENV_DIR'] = directory_name
    env['CM_VIRTUAL_ENV_PATH'] = os.path.join(os.getcwd(), directory_name)

    s = 'Scripts' if os_info['platform'] == 'windows' else 'bin'
    env['CM_VIRTUAL_ENV_SCRIPTS_PATH'] = os.path.join(
        env['CM_VIRTUAL_ENV_PATH'], s)

    env['CM_TMP_PATH'] = env['CM_VIRTUAL_ENV_SCRIPTS_PATH']
    env['CM_TMP_FAIL_IF_NOT_FOUND'] = 'yes'

    r = automation.update_deps({'deps': meta['post_deps'],
                                'update_deps': {'register-python':
                                                {'extra_cache_tags': ','.join(add_python_extra_cache_tags)}}})
    if r['return'] > 0:
        return r

    env['CM_PYTHON_INSTALLED_PATH'] = env['CM_VIRTUAL_ENV_SCRIPTS_PATH']

    return {'return': 0, 'add_extra_cache_tags': add_extra_cache_tags}


def postprocess(i):

    os_info = i['os_info']

    env = i['env']

    state = i['state']

    script_prefix = state.get('script_prefix', [])

    path_to_activate = os.path.join(
        env['CM_VIRTUAL_ENV_SCRIPTS_PATH'], 'activate')

    # If windows, download here otherwise use run.sh
    if os_info['platform'] == 'windows':
        path_to_activate += '.bat'

    s = os_info['run_bat'].replace('${bat_file}', '"' + path_to_activate + '"')

    script_prefix.append(s)
    state['script_prefix'] = script_prefix

    python_name = 'python.exe' if os_info['platform'] == 'windows' else 'python3'

    # Will be passed to get-python to finalize registering of the new python
    env['CM_PYTHON_BIN_WITH_PATH'] = os.path.join(
        env['CM_PYTHON_INSTALLED_PATH'], python_name)

    return {'return': 0}



"""
Original code was extended by Grigori Fursin to support cognata data set
"""

import json
import logging
import os
import time

import cv2
from PIL import Image

import numpy as np
from pycocotools.cocoeval import COCOeval
# import pycoco
import dataset

logging.basicConfig(level=logging.INFO)
log = logging.getLogger("cognata")


class Cognata(dataset.Dataset):
    def __init__(self, data_path, image_list, name, use_cache=0, image_size=None,
                 image_format="NHWC", pre_process=None, count=None, cache_dir=None, preprocessed_dir=None, use_label_map=False, threads=os.cpu_count(),
                 model_config=None, model_num_classes=None, model_image_size=None):  # For ABTF
        super().__init__()

        self.image_size = image_size
        self.image_list = []
        self.label_list = []
        self.image_ids = []
        self.image_sizes = []
        self.count = count
        self.use_cache = use_cache
        self.data_path = data_path
        self.pre_process = pre_process
        self.use_label_map = use_label_map

        self.model_config = model_config
        self.model_num_classes = model_num_classes
        self.model_image_size = model_image_size
        self.ignore_classes = None
        self.files = None
        self.dboxes = None
        self.transform = None
        self.label_map = None
        self.label_info = None
        self.image_bin = []
        self.encoder = None
        self.targets = []

        #######################################################################
        # From ABTF source

        import torch
        from src.utils import generate_dboxes, Encoder
        from src.transform import SSDTransformer
        from src.dataset import prepare_cognata
        import cognata_labels
        import csv
        import ast

        self.dboxes = generate_dboxes(model_config.model, model="ssd")
        self.transform = SSDTransformer(
            self.dboxes, self.model_image_size, val=True)
        self.encoder = Encoder(self.dboxes)

        folders = model_config.dataset['folders']
        cameras = model_config.dataset['cameras']
        self.ignore_classes = [2, 25, 31]
        if 'ignore_classes' in model_config.dataset:
            self.ignore_classes = model_config.dataset['ignore_classes']

        # Grigori added for tests
        # Check if overridden by extrnal environment for tests
        x = os.environ.get(
            'CM_DATASET_MLCOMMONS_COGNATA_SERIAL_NUMBERS',
            '').strip()
        if x != '':
            folders = x.split(';') if ';' in x else [x]

        x = os.environ.get(
            'CM_DATASET_MLCOMMONS_COGNATA_GROUP_NAMES',
            '').strip()
        if x != '':
            cameras = x.split(';') if ';' in x else [x]

        print('')
        print('Cognata folders: {}'.format(str(folders)))
        print('Cognata cameras: {}'.format(str(cameras)))
        print('')

        # From ABTF source
        print('')
        print('Scanning Cognata dataset ...')
        start = time.time()
        files, label_map, label_info = prepare_cognata(
            data_path, folders, cameras, self.ignore_classes)

        self.files = files

        print('  Number of files found: {}'.format(len(files)))
        print('  Time: {:.2f} sec.'.format(time.time() - start))

        if os.environ.get(
                'CM_ABTF_ML_MODEL_TRAINING_FORCE_COGNATA_LABELS', '') == 'yes':
            label_map = cognata_labels.label_map
            label_info = cognata_labels.label_info

        self.label_map = label_map
        self.label_info = label_info

        if self.model_num_classes is not None:
            self.model_num_classes = len(label_map.keys())

        print('')
        print('Preloading and preprocessing Cognata dataset on the fly ...')

        start = time.time()

        idx = 0

        for f in self.files:

            image_name = self.files[idx]['img']

            img = Image.open(image_name).convert('RGB')

            width, height = img.size
            boxes = []
            boxes2 = []
            labels = []
            gt_boxes = []
            targets = []
            with open(self.files[idx]['ann']) as f:
                reader = csv.reader(f)
                rows = list(reader)
                header = rows[0]
                annotations = rows[1:]
                bbox_index = header.index('bounding_box_2D')
                class_index = header.index('object_class')
                distance_index = header.index('center_distance')
                for annotation in annotations:
                    bbox = annotation[bbox_index]
                    bbox = ast.literal_eval(bbox)
                    object_width = bbox[2] - bbox[0]
                    object_height = bbox[3] - bbox[1]
                    object_area = object_width * object_height
                    label = ast.literal_eval(annotation[class_index])
                    distance = ast.literal_eval(annotation[distance_index])
                    if object_area < 50 or int(
                            label) in self.ignore_classes or object_height < 8 or object_width < 8 or distance > 300:
                        continue
                    label = self.label_map[label]
                    boxes.append([bbox[0] / width, bbox[1] / height,
                                 bbox[2] / width, bbox[3] / height])
                    boxes2.append([bbox[0], bbox[1], bbox[2], bbox[3]])
                    gt_boxes.append(
                        [bbox[0], bbox[1], bbox[2], bbox[3], label, 0, 0])
                    labels.append(label)

                boxes = torch.tensor(boxes)
                boxes2 = torch.tensor(boxes2)
                labels = torch.tensor(labels)
                gt_boxes = torch.tensor(gt_boxes)

                targets.append({'boxes': boxes2.to(device='cpu'),
                                'labels': labels.to(device='cpu',
                                                    dtype=torch.int32)})

            img, (height, width), boxes, labels = self.transform(
                img, (height, width), boxes, labels, max_num=500)

            _, height, width = img.shape

            self.image_bin.append(img)
            self.image_ids.append(idx)
            self.image_list.append(image_name)
            self.image_sizes.append((height, width))

            self.label_list.append((labels, boxes))

            self.targets.append(targets)

            # limit the dataset if requested
            idx += 1
            if self.count is not None and idx >= self.count:
                break

        print('  Time: {:.2f} sec.'.format(time.time() - start))
        print('')

        return

    def get_item(self, nr):
        """Get image by number in the list."""

        return self.image_bin[nr], self.label_list[nr]

    def get_item_loc(self, nr):

        return self.files[nr]['img']

    # Grigori added here to be able to return Torch tensor and not Numpy

    def get_samples(self, id_list):

        data = [self.image_list_inmemory[idx] for idx in id_list]
        labels = [self.label_list[idx] for idx in id_list]

        return data, labels


class PostProcessCognata:
    """
    Post processing for tensorflow ssd-mobilenet style models
    """

    def __init__(self):
        self.results = []
        self.good = 0
        self.total = 0
        self.content_ids = []
        self.use_inv_map = False

    def add_results(self, results):
        self.results.extend(results)

    def __call__(self, results, ids, expected=None, result_dict=None, ):

        # Dummy
        processed_results = []
        return processed_results

    def start(self):
        self.results = []
        self.good = 0
        self.total = 0

    def finalize(self, result_dict, ds=None, output_dir=None):

        # To be improved

        from torchmetrics.detection.mean_ap import MeanAveragePrecision
        metric = MeanAveragePrecision(
            iou_type="bbox",
            class_metrics=True,
            backend='faster_coco_eval')

        result_dict["good"] += self.good
        result_dict["total"] += self.total

        preds = []
        targets = []
        # For now batch_size = 1
        for idx in range(0, len(self.results)):
            preds.append(self.results[idx][0])
            id = self.results[idx][0]['id']
            targets.append(ds.targets[id][0])
        metric.update(preds, targets)

        metrics = metric.compute()

        print('=================================================')
        import pprint
        pp = pprint.PrettyPrinter(indent=4)
        pp.pprint(metrics)
        print('=================================================')

        classes = metrics['classes'].tolist()
        map_per_classes = metrics['map_per_class'].tolist()

        final_map = {}
        for c in range(0, len(classes)):
            final_map[ds.label_info[classes[c]]] = float(map_per_classes[c])

        result_dict["mAP"] = float(metrics['map'])
        result_dict["mAP_classes"] = final_map


class PostProcessCognataPt(PostProcessCognata):
    """
    Post processing required by ssd-resnet34 / pytorch
    """

    def __init__(self, nms_threshold, max_output,
                 score_threshold, height, width):
        super().__init__()
        self.nms_threshold = nms_threshold
        self.max_output = max_output
        self.score_threshold = score_threshold
        self.height = height
        self.width = width

    def __call__(self, results, ids, expected=None, result_dict=None):
        # results come as:
        #   detection_boxes,detection_classes,detection_scores

        import torch

        processed_results = []

        # For now 1 result (batch 1) - need to add support for batch size > 1
        # later
        ploc = results[0]
        plabel = results[1]

        # Get predictions (from cognata_eval)
#        ploc, plabel = model(img)
        ploc, plabel = ploc.float(), plabel.float()

        preds = []

        for i in range(ploc.shape[0]):
            dts = []
            labels = []
            scores = []

            ploc_i = ploc[i, :, :].unsqueeze(0)
            plabel_i = plabel[i, :, :].unsqueeze(0)

            result = self.encoder.decode_batch(
                ploc_i, plabel_i, self.nms_threshold, self.max_output)[0]

            loc, label, prob = [r.cpu().numpy() for r in result]
            for loc_, label_, prob_ in zip(loc, label, prob):
                if label_ in expected[i][0]:
                    self.good += 1
                self.total += 1
                dts.append([loc_[0] *
                            self.width, loc_[1] *
                            self.height, loc_[2] *
                            self.width, loc_[3] *
                            self.height,])
                labels.append(label_)
                scores.append(prob_)

            dts = torch.tensor(dts, device='cpu')
            labels = torch.tensor(labels, device='cpu', dtype=torch.int32)
            scores = torch.tensor(scores, device='cpu')
            preds.append({'boxes': dts, 'labels': labels,
                         'scores': scores, 'id': ids[i]})

        # Only batch size supported
        idx = 0

        processed_results.append(preds)

        # self.total += 1

        return processed_results



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil
import stat


def preprocess(i):

    os_info = i['os_info']

    return {'return': 0}


def postprocess(i):

    env = i['env']
    state = i['state']

    if env['CM_HOST_OS_TYPE'].lower() == "windows":
        binary_name = "ptd-windows-x86.exe"
    else:
        binary_name = "ptd-linux-x86"
    if env.get('CM_MLPERF_PTD_PATH', '') == '':
        env['CM_MLPERF_PTD_PATH'] = os.path.join(
            env['CM_MLPERF_POWER_SOURCE'], 'PTD', 'binaries', binary_name)

    file_path = env['CM_MLPERF_PTD_PATH']
    current_permissions = os.stat(file_path).st_mode

    # Check if the file already has execute permissions
    if not (current_permissions & stat.S_IXUSR):  # Check user execute permission
        # Add execute permissions for the user
        os.chmod(file_path, current_permissions | stat.S_IXUSR)

    env['CM_SPEC_PTD_PATH'] = env['CM_MLPERF_PTD_PATH']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import tarfile
import shutil


def preprocess(i):

    recursion_spaces = i['recursion_spaces']

    os_info = i['os_info']

    env = i['env']

    env['CM_TMP_RUN_COPY_SCRIPT'] = "no"

    # If TAR file is not explicitly specified, search
    if env.get('CM_CUDNN_TAR_FILE_PATH', '') == '':

        cuda_path_lib = env.get('CM_CUDA_PATH_LIB')

        if os_info['platform'] == 'windows':
            extra_pre = ''
            extra_ext = 'lib'
        else:
            extra_pre = 'lib'
            extra_ext = 'so'

        libfilename = extra_pre + 'cudnn.' + extra_ext
        env['CM_CUDNN_VERSION'] = 'vdetected'

        if os.path.exists(os.path.join(cuda_path_lib, libfilename)):
            env['CM_CUDA_PATH_LIB_CUDNN'] = env['CM_CUDA_PATH_LIB']
            return {'return': 0}

        if env.get('CM_TMP_PATH', '').strip() != '':
            path = env.get('CM_TMP_PATH')
            if os.path.exists(os.path.join(path, libfilename)):
                env['CM_CUDA_PATH_LIB_CUDNN'] = path
                return {'return': 0}

        if env.get('CM_INPUT', '').strip() == '':
            if os_info['platform'] == 'windows':
                if env.get('CM_TMP_PATH', '').strip() == '':
                    # Check in "C:\Program Files\NVIDIA GPU Computing
                    # Toolkit\CUDA"
                    paths = []
                    for path in ["C:\\Program Files\\NVIDIA GPU Computing Toolkit\\CUDA",
                                 "C:\\Program Files (x86)\\NVIDIA GPU Computing Toolkit\\CUDA"]:
                        if os.path.isdir(path):
                            dirs = os.listdir(path)
                            for dr in dirs:
                                path2 = os.path.join(path, dr, 'lib')
                                if os.path.isdir(path2):
                                    paths.append(path2)

                    if len(paths) > 0:
                        tmp_paths = ';'.join(paths)
                        tmp_paths += ';' + os.environ.get('PATH', '')

                        env['CM_TMP_PATH'] = tmp_paths
                        env['CM_TMP_PATH_IGNORE_NON_EXISTANT'] = 'yes'

            else:
                # paths to cuda are not always in PATH - add a few typical locations to search for
                # (unless forced by a user)

                cm_tmp_path = env.get('CM_TMP_PATH', '').strip()
                if cm_tmp_path != '':
                    cm_tmp_path += ':'
                cm_tmp_path += '/usr/local/cuda/lib64:/usr/cuda/lib64:/usr/local/cuda/lib:/usr/cuda/lib:/usr/local/cuda-11/lib64:/usr/cuda-11/lib:/usr/local/cuda-12/lib:/usr/cuda-12/lib:/usr/local/packages/cuda/lib'
                cm_tmp_path += os.path.expandvars(':$CUDNN_ROOT/lib')
                env['CM_TMP_PATH'] = cm_tmp_path
                env['CM_TMP_PATH_IGNORE_NON_EXISTANT'] = 'yes'

                for lib_path in env.get(
                        '+CM_HOST_OS_DEFAULT_LIBRARY_PATH', []):
                    if (os.path.exists(lib_path)):
                        env['CM_TMP_PATH'] += ':' + lib_path

        r = i['automation'].find_artifact({'file_name': libfilename,
                                           'env': env,
                                           'os_info': os_info,
                                           'default_path_env_key': 'LD_LIBRARY_PATH',
                                           'detect_version': False,
                                           'env_path_key': 'CM_CUDA_PATH_LIB_CUDNN',
                                           'run_script_input': i['run_script_input'],
                                           'recursion_spaces': recursion_spaces})
        if r['return'] > 0:
            if os_info['platform'] == 'windows':
                return r

            if r['return'] == 16:
                env['CM_TMP_REQUIRE_INSTALL'] = "yes"
            else:
                return r
        else:
            # On Linux we may detected file instead of path to cudnn
            if os.path.isfile(env['CM_CUDA_PATH_LIB_CUDNN']):
                env['CM_CUDA_PATH_LIB_CUDNN'] = os.path.dirname(
                    env['CM_CUDA_PATH_LIB_CUDNN'])

            return {'return': 0}

    if env.get('CM_CUDNN_TAR_FILE_PATH', '') == '':
        return {'return': 1, 'error': 'Please envoke cm run script "get cudnn" --tar_file={full path to the cuDNN tar file}'}

    print('Untaring file - can take some time ...')

    my_tar = tarfile.open(os.path.expanduser(env['CM_CUDNN_TAR_FILE_PATH']))
    folder_name = my_tar.getnames()[0]
    if not os.path.exists(os.path.join(os.getcwd(), folder_name)):
        my_tar.extractall()
    my_tar.close()

    import re
    version_match = re.match(r'cudnn-.*?-(\d+.\d+.\d+.\d+)', folder_name)
    if not version_match:
        return {
            'return': 1, 'error': 'Extracted CUDNN folder does not seem proper - Version information missing'}
    version = version_match.group(1)
    env['CM_CUDNN_VERSION'] = version

    inc_path = os.path.join(os.getcwd(), folder_name, "include")
    lib_path = os.path.join(os.getcwd(), folder_name, "lib")
    cuda_inc_path = env['CM_CUDA_PATH_INCLUDE']
    cuda_lib_path = env['CM_CUDA_PATH_LIB']
    env['CM_CUDA_PATH_LIB_CUDNN'] = env['CM_CUDA_PATH_LIB']
    env['CM_CUDA_PATH_INCLUDE_CUDNN'] = env['CM_CUDA_PATH_INCLUDE']

    try:
        print(
            "Copying cudnn include files to {}(CUDA_INCLUDE_PATH)".format(cuda_inc_path))
        shutil.copytree(inc_path, cuda_inc_path, dirs_exist_ok=True)
        print("Copying cudnn lib files to {}CUDA_LIB_PATH".format(cuda_lib_path))
        shutil.copytree(lib_path, cuda_lib_path, dirs_exist_ok=True)
    except BaseException:
        # Need to copy to system path via run.sh
        env['CM_TMP_RUN_COPY_SCRIPT'] = "yes"
        env['CM_TMP_INC_PATH'] = inc_path
        env['CM_TMP_LIB_PATH'] = lib_path

    return {'return': 0}


def postprocess(i):

    os_info = i['os_info']

    env = i['env']

    version = env['CM_CUDNN_VERSION']

    if version == 'vdetected':
        path_to_cudnn = env.get('CM_CUDA_PATH_LIB_CUDNN', '')
        if os.path.isdir(path_to_cudnn):
            path_to_include = path_to_cudnn
            path_to_include_file = ''
            for j in range(0, 2):
                path_to_include = os.path.dirname(path_to_include)
                x = os.path.join(path_to_include, 'include', 'cudnn_version.h')
                if os.path.isfile(x):
                    path_to_include_file = x
                    break

            if path_to_include_file == '' and path_to_cudnn.startswith('/lib'):
                x = os.path.join('/usr', 'include', 'cudnn_version.h')
                if os.path.isfile(x):
                    path_to_include_file = x

            if path_to_include_file != '':
                env['CM_CUDA_PATH_INCLUDE_CUDNN'] = os.path.dirname(
                    path_to_include_file)

                r = utils.load_txt(path_to_include_file, split=True)
                if r['return'] == 0:
                    lst = r['list']

                    xversion = ''

                    for l in lst:
                        l = l.strip()

                        x = '#define CUDNN_MAJOR '
                        if l.startswith(x):
                            xversion = l[len(x):]

                        x = '#define CUDNN_MINOR '
                        if l.startswith(x):
                            xversion += '.' + l[len(x):]

                        x = '#define CUDNN_PATCHLEVEL '
                        if l.startswith(x):
                            xversion += '.' + l[len(x):]

                    if xversion != '':
                        version = xversion
                        env['CM_CUDNN_VERSION'] = xversion

    env['CM_CUDA_PATH_LIB_CUDNN_EXISTS'] = 'yes'

    return {'return': 0, 'version': version}



import os

import torch

if __name__ == "__main__":

    print('')
    print('Main script:')
    print('ENV CM_VAR1: {}'.format(os.environ.get('CM_VAR1', '')))
    print('ENV USE_CUDA: {}'.format(os.environ.get('USE_CUDA', '')))
    print('')
    print('PyTorch version: {}'.format(torch.__version__))
    print('')

    exit(0)
